@@ -14,10 +14,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include "avformat.h"
-#include "avi.h"
+#include "riff.h"
 
 //#define DEBUG
 //#define DEBUG_DUMP_INDEX // XXX dumbdriving-271.nsv breaks with it commented!!
@@ -46,8 +46,8 @@
 
 /*
  * notes on the header (Francois Revol):
- * 
- * It is followed by strings, then a table, but nothing tells 
+ *
+ * It is followed by strings, then a table, but nothing tells
  * where the table begins according to (1). After checking faster.nsv,
  * I believe NVSf[16-19] gives the size of the strings data
  * (that is the offset of the data table after the header).
@@ -56,15 +56,15 @@
  * Then, about NSVf[12-15], faster.nsf has 179700. When veiwing it in VLC,
  * I noticed there was about 1 NVSs chunk/s, so I ran
  * strings faster.nsv | grep NSVs | wc -l
- * which gave me 180. That leads me to think that NSVf[12-15] might be the 
+ * which gave me 180. That leads me to think that NSVf[12-15] might be the
  * file length in milliseconds.
  * Let's try that:
  * for f in *.nsv; do HTIME="$(od -t x4 "$f" | head -1 | sed 's/.* //')"; echo "'$f' $((0x$HTIME))s = $((0x$HTIME/1000/60)):$((0x$HTIME/1000%60))"; done
  * except for nstrailer (which doesn't have an NSVf header), it repports correct time.
  *
- * nsvtrailer.nsv (S1) does not have any NSVf header, only NSVs chunks, 
+ * nsvtrailer.nsv (S1) does not have any NSVf header, only NSVs chunks,
  * so the header seems to not be mandatory. (for streaming).
- * 
+ *
  * index slice duration check (excepts nsvtrailer.nsv):
  * for f in [^n]*.nsv; do DUR="$(ffmpeg -i "$f" 2>/dev/null | grep 'NSVf duration' | cut -d ' ' -f 4)"; IC="$(ffmpeg -i "$f" 2>/dev/null | grep 'INDEX ENTRIES' | cut -d ' ' -f 2)"; echo "duration $DUR, slite time $(($DUR/$IC))"; done
  */
@@ -80,7 +80,7 @@
 #ifdef DEBUG
 #define PRINT(_v) printf _v
 #else
-#define PRINT(_v) 
+#define PRINT(_v)
 #endif
 
 #if 0
@@ -156,10 +156,10 @@ typedef struct NSVStream {
     int frame_offset; /* current frame (video) or byte (audio) counter
                          (used to compute the pts) */
     int scale;
-    int rate;    
+    int rate;
     int sample_size; /* audio only data */
     int start;
-    
+
     int new_frame_offset; /* temporary storage (used during seek) */
     int cum_len; /* temporary storage (used during seek) */
 } NSVStream;
@@ -205,13 +205,13 @@ static const CodecTag nsv_codec_audio_tags[] = {
     { 0, 0 },
 };
 
-static const uint64_t nsv_framerate_table[] = {
-    ((uint64_t)AV_TIME_BASE * 30),
-    ((uint64_t)AV_TIME_BASE * 30000 / 1001), /* 29.97 */
-    ((uint64_t)AV_TIME_BASE * 25),
-    ((uint64_t)AV_TIME_BASE * 24000 / 1001), /* 23.98 */
-    ((uint64_t)AV_TIME_BASE * 30), /* ?? */
-    ((uint64_t)AV_TIME_BASE * 15000 / 1001), /* 14.98 */
+static const AVRational nsv_framerate_table[] = {
+    {30,1},
+    {30000,1001},
+    {25,1},
+    {24000,1001},
+    {30,1},
+    {15000,1001},
 };
 
 //static int nsv_load_index(AVFormatContext *s);
@@ -235,11 +235,11 @@ static int nsv_resync(AVFormatContext *s)
     ByteIOContext *pb = &s->pb;
     uint32_t v = 0;
     int i;
-    
-    PRINT(("%s(), offset = %Ld, state = %d\n", __FUNCTION__, url_ftell(pb), nsv->state));
-    
+
+    PRINT(("%s(), offset = %"PRId64", state = %d\n", __FUNCTION__, url_ftell(pb), nsv->state));
+
     //nsv->state = NSV_UNSYNC;
-    
+
     for (i = 0; i < NSV_MAX_RESYNC; i++) {
         if (url_feof(pb)) {
             PRINT(("NSV EOF\n"));
@@ -253,7 +253,7 @@ static int nsv_resync(AVFormatContext *s)
             PRINT(("NSV resync: [%d] = %02x\n", i, v & 0x0FF));
         }
 */
-        
+
         if ((v & 0x0000ffff) == 0xefbe) { /* BEEF */
             PRINT(("NSV resynced on BEEF after %d bytes\n", i+1));
             nsv->state = NSV_FOUND_BEEF;
@@ -270,7 +270,7 @@ static int nsv_resync(AVFormatContext *s)
             nsv->state = NSV_FOUND_NSVS;
             return 0;
         }
-        
+
     }
     PRINT(("NSV sync lost\n"));
     return -1;
@@ -280,21 +280,16 @@ static int nsv_parse_NSVf_header(AVFormatContext *s, AVFormatParameters *ap)
 {
     NSVContext *nsv = s->priv_data;
     ByteIOContext *pb = &s->pb;
-    uint32_t tag, tag1, handler;
-    int codec_type, stream_index, frame_period, bit_rate, scale, rate;
-    unsigned int file_size, size, nb_frames;
+    unsigned int file_size, size;
     int64_t duration;
     int strings_size;
     int table_entries;
     int table_entries_used;
-    int i, n;
-    AVStream *st;
-    NSVStream *ast;
 
     PRINT(("%s()\n", __FUNCTION__));
 
     nsv->state = NSV_UNSYNC; /* in case we fail */
-    
+
     size = get_le32(pb);
     if (size < 28)
         return -1;
@@ -302,23 +297,22 @@ static int nsv_parse_NSVf_header(AVFormatContext *s, AVFormatParameters *ap)
 
     //s->file_size = (uint32_t)get_le32(pb);
     file_size = (uint32_t)get_le32(pb);
-    PRINT(("NSV NSVf chunk_size %ld\n", size));
-    PRINT(("NSV NSVf file_size %Ld\n", file_size));
+    PRINT(("NSV NSVf chunk_size %u\n", size));
+    PRINT(("NSV NSVf file_size %u\n", file_size));
 
-    duration = get_le32(pb); /* in ms */
-    nsv->duration = duration * AV_TIME_BASE / 1000; /* convert */
-    PRINT(("NSV NSVf duration %Ld ms\n", duration));
+    nsv->duration = duration = get_le32(pb); /* in ms */
+    PRINT(("NSV NSVf duration %"PRId64" ms\n", duration));
     // XXX: store it in AVStreams
 
     strings_size = get_le32(pb);
     table_entries = get_le32(pb);
     table_entries_used = get_le32(pb);
-    PRINT(("NSV NSVf info-strings size: %d, table entries: %d, bis %d\n", 
+    PRINT(("NSV NSVf info-strings size: %d, table entries: %d, bis %d\n",
             strings_size, table_entries, table_entries_used));
     if (url_feof(pb))
         return -1;
-    
-    PRINT(("NSV got header; filepos %Ld\n", url_ftell(pb)));
+
+    PRINT(("NSV got header; filepos %"PRId64"\n", url_ftell(pb)));
 
     if (strings_size > 0) {
         char *strings; /* last byte will be '\0' to play safe with str*() */
@@ -360,19 +354,20 @@ static int nsv_parse_NSVf_header(AVFormatContext *s, AVFormatParameters *ap)
     }
     if (url_feof(pb))
         return -1;
-    
-    PRINT(("NSV got infos; filepos %Ld\n", url_ftell(pb)));
+
+    PRINT(("NSV got infos; filepos %"PRId64"\n", url_ftell(pb)));
 
     if (table_entries_used > 0) {
         nsv->index_entries = table_entries_used;
         if((unsigned)table_entries >= UINT_MAX / sizeof(uint32_t))
             return -1;
         nsv->nsvf_index_data = av_malloc(table_entries * sizeof(uint32_t));
-        get_buffer(pb, nsv->nsvf_index_data, table_entries * sizeof(uint32_t));
+#warning "FIXME: Byteswap buffer as needed"
+        get_buffer(pb, (unsigned char *)nsv->nsvf_index_data, table_entries * sizeof(uint32_t));
     }
 
-    PRINT(("NSV got index; filepos %Ld\n", url_ftell(pb)));
-    
+    PRINT(("NSV got index; filepos %"PRId64"\n", url_ftell(pb)));
+
 #ifdef DEBUG_DUMP_INDEX
 #define V(v) ((v<0x20 || v > 127)?'.':v)
     /* dump index */
@@ -385,15 +380,15 @@ static int nsv_parse_NSVf_header(AVFormatContext *s, AVFormatParameters *ap)
         PRINT(("NSV [0x%08lx][0x%08lx]: %02x %02x %02x %02x %02x %02x %02x %02x"
            "%c%c%c%c%c%c%c%c\n",
            nsv->nsvf_index_data[i], size + nsv->nsvf_index_data[i],
-           b[0], b[1], b[2], b[3], b[4], b[5], b[6], b[7], 
+           b[0], b[1], b[2], b[3], b[4], b[5], b[6], b[7],
            V(b[0]), V(b[1]), V(b[2]), V(b[3]), V(b[4]), V(b[5]), V(b[6]), V(b[7]) ));
     }
     //url_fseek(pb, size, SEEK_SET); /* go back to end of header */
 #undef V
 #endif
-    
+
     url_fseek(pb, nsv->base_offset + size, SEEK_SET); /* required for dumbdriving-271.nsv (2 extra bytes) */
-    
+
     if (url_feof(pb))
         return -1;
     nsv->state = NSV_HAS_READ_NSVF;
@@ -406,7 +401,8 @@ static int nsv_parse_NSVs_header(AVFormatContext *s, AVFormatParameters *ap)
     ByteIOContext *pb = &s->pb;
     uint32_t vtag, atag;
     uint16_t vwidth, vheight;
-    uint32_t framerate;
+    AVRational framerate;
+    int i;
     uint16_t unknown;
     AVStream *st;
     NSVStream *nst;
@@ -416,19 +412,19 @@ static int nsv_parse_NSVs_header(AVFormatContext *s, AVFormatParameters *ap)
     atag = get_le32(pb);
     vwidth = get_le16(pb);
     vheight = get_le16(pb);
-    framerate = (uint8_t)get_byte(pb);
+    i = get_byte(pb);
     /* XXX how big must the table be ? */
     /* seems there is more to that... */
-    PRINT(("NSV NSVs framerate code %2x\n", framerate));
-    framerate = (framerate & 0x80)?(nsv_framerate_table[framerate & 0x7F]):(framerate*AV_TIME_BASE);
+    PRINT(("NSV NSVs framerate code %2x\n", i));
+    if(i&0x80) framerate= nsv_framerate_table[i & 0x7F];
+    else       framerate= (AVRational){i, 1};
     unknown = get_le16(pb);
 #ifdef DEBUG
     print_tag("NSV NSVs vtag", vtag, 0);
     print_tag("NSV NSVs atag", atag, 0);
     PRINT(("NSV NSVs vsize %dx%d\n", vwidth, vheight));
-    PRINT(("NSV NSVs framerate %2x\n", framerate));
 #endif
-    
+
     /* XXX change to ap != NULL ? */
     if (s->nb_streams == 0) { /* streams not yet published, let's do that */
         nsv->vtag = vtag;
@@ -444,18 +440,16 @@ static int nsv_parse_NSVs_header(AVFormatContext *s, AVFormatParameters *ap)
             if (!nst)
                 goto fail;
             st->priv_data = nst;
-            st->codec.codec_type = CODEC_TYPE_VIDEO;
-            st->codec.codec_tag = vtag;
-            st->codec.codec_id = codec_get_id(nsv_codec_video_tags, vtag);
-            st->codec.width = vwidth;
-            st->codec.height = vheight;
-            st->codec.bits_per_sample = 24; /* depth XXX */
+            st->codec->codec_type = CODEC_TYPE_VIDEO;
+            st->codec->codec_tag = vtag;
+            st->codec->codec_id = codec_get_id(nsv_codec_video_tags, vtag);
+            st->codec->width = vwidth;
+            st->codec->height = vheight;
+            st->codec->bits_per_sample = 24; /* depth XXX */
 
-            st->codec.frame_rate = framerate;
-            st->codec.frame_rate_base = AV_TIME_BASE;
-            av_set_pts_info(st, 64, AV_TIME_BASE, framerate);
+            av_set_pts_info(st, 64, framerate.den, framerate.num);
             st->start_time = 0;
-            st->duration = nsv->duration;
+            st->duration = av_rescale(nsv->duration, framerate.num, 1000*framerate.den);
         }
         if (atag != T_NONE) {
 #ifndef DISABLE_AUDIO
@@ -467,17 +461,17 @@ static int nsv_parse_NSVs_header(AVFormatContext *s, AVFormatParameters *ap)
             if (!nst)
                 goto fail;
             st->priv_data = nst;
-            st->codec.codec_type = CODEC_TYPE_AUDIO;
-            st->codec.codec_tag = atag;
-            st->codec.codec_id = codec_get_id(nsv_codec_audio_tags, atag);
+            st->codec->codec_type = CODEC_TYPE_AUDIO;
+            st->codec->codec_tag = atag;
+            st->codec->codec_id = codec_get_id(nsv_codec_audio_tags, atag);
             st->start_time = 0;
-            st->duration = nsv->duration;
-            
+//            st->duration = nsv->duration; //FIXME
+
             st->need_parsing = 1; /* for PCM we will read a chunk later and put correct info */
             /* XXX:FIXME */
-            //st->codec.channels = 2; //XXX:channels;
-            //st->codec.sample_rate = 1000;
-            //av_set_pts_info(st, 64, 1, st->codec.sample_rate);
+            //st->codec->channels = 2; //XXX:channels;
+            //st->codec->sample_rate = 1000;
+            //av_set_pts_info(st, 64, 1, st->codec->sample_rate);
 
 #endif
         }
@@ -501,21 +495,14 @@ fail:
 static int nsv_read_header(AVFormatContext *s, AVFormatParameters *ap)
 {
     NSVContext *nsv = s->priv_data;
-    ByteIOContext *pb = &s->pb;
-    uint32_t tag, tag1, handler;
-    int codec_type, stream_index, frame_period, bit_rate, scale, rate;
-    unsigned int size, nb_frames;
-    int table_entries;
-    int i, n, err;
-    AVStream *st;
-    NSVStream *ast;
+    int i, err;
 
     PRINT(("%s()\n", __FUNCTION__));
     PRINT(("filename '%s'\n", s->filename));
 
     nsv->state = NSV_UNSYNC;
     nsv->ahead[0].data = nsv->ahead[1].data = NULL;
-    
+
     for (i = 0; i < NSV_MAX_RESYNC_TRIES; i++) {
         if (nsv_resync(s) < 0)
             return -1;
@@ -531,7 +518,7 @@ static int nsv_read_header(AVFormatContext *s, AVFormatParameters *ap)
         return -1;
     /* now read the first chunk, so we can attempt to decode more info */
     err = nsv_read_chunk(s, 1);
-    
+
     PRINT(("parsed header\n"));
     return 0;
 }
@@ -543,23 +530,22 @@ static int nsv_read_chunk(AVFormatContext *s, int fill_header)
     AVStream *st[2] = {NULL, NULL};
     NSVStream *nst;
     AVPacket *pkt;
-    uint32_t v = 0;
     int i, err = 0;
     uint8_t auxcount; /* number of aux metadata, also 4 bits of vsize */
     uint32_t vsize;
     uint16_t asize;
     uint16_t auxsize;
     uint32_t auxtag;
-    
+
     PRINT(("%s(%d)\n", __FUNCTION__, fill_header));
-    
+
     if (nsv->ahead[0].data || nsv->ahead[1].data)
         return 0; //-1; /* hey! eat what you've in your plate first! */
 
 null_chunk_retry:
     if (url_feof(pb))
         return -1;
-    
+
     for (i = 0; i < NSV_MAX_RESYNC_TRIES && nsv->state < NSV_FOUND_NSVS && !err; i++)
         err = nsv_resync(s);
     if (err < 0)
@@ -570,45 +556,44 @@ null_chunk_retry:
         return err;
     if (nsv->state != NSV_HAS_READ_NSVS && nsv->state != NSV_FOUND_BEEF)
         return -1;
-    
+
     auxcount = get_byte(pb);
     vsize = get_le16(pb);
     asize = get_le16(pb);
     vsize = (vsize << 4) | (auxcount >> 4);
     auxcount &= 0x0f;
-    PRINT(("NSV CHUNK %d aux, %ld bytes video, %d bytes audio\n", auxcount, vsize, asize));
+    PRINT(("NSV CHUNK %d aux, %u bytes video, %d bytes audio\n", auxcount, vsize, asize));
     /* skip aux stuff */
     for (i = 0; i < auxcount; i++) {
         auxsize = get_le16(pb);
         auxtag = get_le32(pb);
-        PRINT(("NSV aux data: '%c%c%c%c', %d bytes\n", 
-              (auxtag & 0x0ff), 
-              ((auxtag >> 8) & 0x0ff), 
+        PRINT(("NSV aux data: '%c%c%c%c', %d bytes\n",
+              (auxtag & 0x0ff),
+              ((auxtag >> 8) & 0x0ff),
               ((auxtag >> 16) & 0x0ff),
               ((auxtag >> 24) & 0x0ff),
               auxsize));
         url_fskip(pb, auxsize);
         vsize -= auxsize + sizeof(uint16_t) + sizeof(uint32_t); /* that's becoming braindead */
     }
-    
+
     if (url_feof(pb))
         return -1;
     if (!vsize && !asize) {
         nsv->state = NSV_UNSYNC;
         goto null_chunk_retry;
     }
-    
+
     /* map back streams to v,a */
     if (s->streams[0])
         st[s->streams[0]->id] = s->streams[0];
     if (s->streams[1])
         st[s->streams[1]->id] = s->streams[1];
-    
+
     if (vsize/* && st[NSV_ST_VIDEO]*/) {
         nst = st[NSV_ST_VIDEO]->priv_data;
         pkt = &nsv->ahead[NSV_ST_VIDEO];
-        av_new_packet(pkt, vsize);
-        get_buffer(pb, pkt->data, vsize);
+        av_get_packet(pb, pkt, vsize);
         pkt->stream_index = st[NSV_ST_VIDEO]->index;//NSV_ST_VIDEO;
         pkt->dts = nst->frame_offset++;
         pkt->flags |= PKT_FLAG_KEY; /* stupid format has no way to tell XXX: try the index */
@@ -622,7 +607,7 @@ null_chunk_retry:
         pkt = &nsv->ahead[NSV_ST_AUDIO];
         /* read raw audio specific header on the first audio chunk... */
         /* on ALL audio chunks ?? seems so! */
-        if (asize && st[NSV_ST_AUDIO]->codec.codec_tag == MKTAG('P', 'C', 'M', ' ')/* && fill_header*/) {
+        if (asize && st[NSV_ST_AUDIO]->codec->codec_tag == MKTAG('P', 'C', 'M', ' ')/* && fill_header*/) {
             uint8_t bps;
             uint8_t channels;
             uint16_t samplerate;
@@ -630,7 +615,7 @@ null_chunk_retry:
             channels = get_byte(pb);
             samplerate = get_le16(pb);
             asize-=4;
-            PRINT(("NSV RAWAUDIO: bps %d, nchan %d, srate %ld\n", bps, channels, samplerate));
+            PRINT(("NSV RAWAUDIO: bps %d, nchan %d, srate %d\n", bps, channels, samplerate));
             if (fill_header) {
                 st[NSV_ST_AUDIO]->need_parsing = 0; /* we know everything */
                 if (bps != 16) {
@@ -638,26 +623,24 @@ null_chunk_retry:
                 }
                 bps /= channels; // ???
                 if (bps == 8)
-                    st[NSV_ST_AUDIO]->codec.codec_id = CODEC_ID_PCM_U8;
+                    st[NSV_ST_AUDIO]->codec->codec_id = CODEC_ID_PCM_U8;
                 samplerate /= 4;/* UGH ??? XXX */
                 channels = 1;
-                st[NSV_ST_AUDIO]->codec.channels = channels;
-                st[NSV_ST_AUDIO]->codec.sample_rate = samplerate;
-                av_set_pts_info(st[NSV_ST_AUDIO], 64, 1, 
-                                st[NSV_ST_AUDIO]->codec.sample_rate);
-                PRINT(("NSV RAWAUDIO: bps %d, nchan %d, srate %ld\n", bps, channels, samplerate));
+                st[NSV_ST_AUDIO]->codec->channels = channels;
+                st[NSV_ST_AUDIO]->codec->sample_rate = samplerate;
+                av_set_pts_info(st[NSV_ST_AUDIO], 64, 1,
+                                st[NSV_ST_AUDIO]->codec->sample_rate);
+                PRINT(("NSV RAWAUDIO: bps %d, nchan %d, srate %d\n", bps, channels, samplerate));
             }
         }
-        av_new_packet(pkt, asize);
-        if (asize)
-            get_buffer(pb, pkt->data, asize);
+        av_get_packet(pb, pkt, asize);
         pkt->stream_index = st[NSV_ST_AUDIO]->index;//NSV_ST_AUDIO;
         //pkt->dts = nst->frame_offset;
         //if (nst->sample_size)
         //    pkt->dts /= nst->sample_size;
         nst->frame_offset += asize; // XXX: that's valid only for PCM !?
     }
-    
+
     //pkt->flags |= PKT_FLAG_KEY;
     nsv->state = NSV_UNSYNC;
     return 0;
@@ -667,46 +650,47 @@ null_chunk_retry:
 static int nsv_read_packet(AVFormatContext *s, AVPacket *pkt)
 {
     NSVContext *nsv = s->priv_data;
-    ByteIOContext *pb = &s->pb;
     int i, err = 0;
 
     PRINT(("%s()\n", __FUNCTION__));
-    
+
     /* in case we don't already have something to eat ... */
     if (nsv->ahead[0].data == NULL && nsv->ahead[1].data == NULL)
         err = nsv_read_chunk(s, 0);
     if (err < 0)
         return err;
-    
+
     /* now pick one of the plates */
     for (i = 0; i < 2; i++) {
         if (nsv->ahead[i].data) {
-        	PRINT(("%s: using cached packet[%d]\n", __FUNCTION__, i));
+                PRINT(("%s: using cached packet[%d]\n", __FUNCTION__, i));
             /* avoid the cost of new_packet + memcpy(->data) */
             memcpy(pkt, &nsv->ahead[i], sizeof(AVPacket));
             nsv->ahead[i].data = NULL; /* we ate that one */
             return pkt->size;
         }
     }
-    
+
     /* this restaurant is not approvisionned :^] */
     return -1;
 }
 
 static int nsv_read_seek(AVFormatContext *s, int stream_index, int64_t timestamp, int flags)
 {
+#if 0
     NSVContext *avi = s->priv_data;
     AVStream *st;
     NSVStream *ast;
     int frame_number, i;
     int64_t pos;
+#endif
 
     return -1;
 }
 
 static int nsv_read_close(AVFormatContext *s)
 {
-    int i;
+/*     int i; */
     NSVContext *nsv = s->priv_data;
 
     if (nsv->index_entries)
@@ -721,8 +705,7 @@ static int nsv_read_close(AVFormatContext *s)
             av_free(ast->index_entries);
             av_free(ast);
         }
-        av_free(st->codec.extradata);
-        av_free(st->codec.palctrl);
+        av_free(st->codec->palctrl);
     }
 
 #endif
@@ -759,7 +742,7 @@ static int nsv_probe(AVProbeData *p)
     return 0;
 }
 
-static AVInputFormat nsv_iformat = {
+AVInputFormat nsv_demuxer = {
     "nsv",
     "NullSoft Video format",
     sizeof(NSVContext),
@@ -769,9 +752,3 @@ static AVInputFormat nsv_iformat = {
     nsv_read_close,
     nsv_read_seek,
 };
-
-int nsvdec_init(void)
-{
-    av_register_input_format(&nsv_iformat);
-    return 0;
-}
