@@ -22,8 +22,8 @@
 #include <iostream>
 using namespace std;
 
-#ifdef USE_LIRC
 #include <pthread.h>
+#ifdef USE_LIRC
 #include "lirc.h"
 #include "lircevent.h"
 #endif
@@ -42,816 +42,6 @@ using namespace std;
 #include "screensaver.h"
 #include "mythdbcon.h"
 
-#ifdef USE_LIRC
-static void *SpawnLirc(void *param)
-{
-    MythMainWindow *main_window = (MythMainWindow *)param;
-    QString config_file = MythContext::GetConfDir() + "/lircrc";
-    QString program("mythtv");
-    LircClient *cl = new LircClient(main_window);
-    if (!cl->Init(config_file, program))
-        cl->Process();
-
-    return NULL;
-}
-#endif
-
-#ifdef USE_JOYSTICK_MENU
-static void *SpawnJoystickMenu(void *param)
-{
-    MythMainWindow *main_window = (MythMainWindow *)param;
-    QString config_file = MythContext::GetConfDir() + "/joystickmenurc";
-    JoystickMenuClient *js = new JoystickMenuClient(main_window);
-    if (!js->Init(config_file))
-        js->Process();
-
-    return NULL;
-}
-#endif
-
-class KeyContext
-{
-  public:
-    void AddMapping(int key, QString action)
-    {
-        actionMap[key].append(action);
-    }
-
-    bool GetMapping(int key, QStringList &actions)
-    {
-        if (actionMap.count(key) > 0)
-        {
-            actions += actionMap[key];
-            return true;
-        }
-        return false;
-    }
-
-    QMap<int, QStringList> actionMap;
-};
-
-struct JumpData
-{
-    void (*callback)(void);
-    QString destination;
-    QString description;
-};
-
-struct MHData
-{
-    void (*callback)(MythMediaDevice *mediadevice);
-    int MediaType;
-    QString destination;
-    QString description;
-};
-
-struct MPData {
-    QString description;
-    MediaPlayCallback playFn;
-};
-
-class MythMainWindowPrivate
-{
-  public:
-    int TranslateKeyNum(QKeyEvent *e);
-
-    float wmult, hmult;
-    int screenwidth, screenheight;
-    int xbase, ybase;
-    bool does_fill_screen;
-
-    vector<QWidget *> widgetList;
-
-#ifdef USE_JOYSTICK_MENU
-    bool ignore_joystick_keys;
-#endif
-
-#ifdef USE_LIRC
-    bool ignore_lirc_keys;
-#endif
-
-    bool exitingtomain;
-
-    QDict<KeyContext> keyContexts;
-    QMap<int, JumpData*> jumpMap;
-    QMap<QString, JumpData> destinationMap;
-    QMap<QString, MHData> mediaHandlerMap;
-    QMap<QString, MPData> mediaPluginMap;
-
-    void (*exitmenucallback)(void);
-
-    void (*exitmenumediadevicecallback)(MythMediaDevice* mediadevice);
-    MythMediaDevice * mediadeviceforcallback;
-
-    int escapekey;
-};
-
-// Make keynum in QKeyEvent be equivalent to what's in QKeySequence
-int MythMainWindowPrivate::TranslateKeyNum(QKeyEvent* e)
-{
-    int keynum = e->key();
-
-    if (keynum != Qt::Key_Escape &&
-        (keynum <  Qt::Key_Shift || keynum > Qt::Key_ScrollLock))
-    {
-        Qt::ButtonState modifiers;
-        // if modifiers have been pressed, rebuild keynum
-        if ((modifiers = e->state()) != 0)
-        {
-            int modnum = (((modifiers & Qt::ShiftButton) &&
-                           keynum > 0x7f) ? Qt::SHIFT : 0) |
-                         ((modifiers & Qt::ControlButton) ? Qt::CTRL : 0) |
-                         ((modifiers & Qt::MetaButton) ? Qt::META : 0) |
-                         ((modifiers & Qt::AltButton) ? Qt::ALT : 0);
-            modnum &= ~Qt::UNICODE_ACCEL;
-            return (keynum |= modnum);
-        }
-    }
-
-    return keynum;
-}
-
-MythMainWindow::MythMainWindow(QWidget *parent, const char *name, bool modal, 
-                               WFlags flags)
-              : QDialog(parent, name, modal, flags)
-{
-    d = new MythMainWindowPrivate;
-
-    Init();
-    d->exitingtomain = false;
-    d->exitmenucallback = false;
-    d->exitmenumediadevicecallback = false;
-    d->mediadeviceforcallback = NULL;
-    d->escapekey = Key_Escape;
-
-#ifdef USE_LIRC
-    d->ignore_lirc_keys = false;
-    pthread_t lirc_tid;
-    pthread_attr_t attr;
-    pthread_attr_init(&attr);
-    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-
-    pthread_create(&lirc_tid, &attr, SpawnLirc, this);
-#endif
-
-#ifdef USE_JOYSTICK_MENU
-    d->ignore_joystick_keys = false;
-    pthread_t js_tid;
-
-    pthread_attr_t attr2;
-    pthread_attr_init(&attr2);
-    pthread_attr_setdetachstate(&attr2, PTHREAD_CREATE_DETACHED);
-
-    pthread_create(&js_tid, &attr2, SpawnJoystickMenu, this);
-#endif
-
-    d->keyContexts.setAutoDelete(true);
-
-    RegisterKey("Global", "UP", "Up Arrow", "Up");
-    RegisterKey("Global", "DOWN", "Down Arrow", "Down");
-    RegisterKey("Global", "LEFT", "Left Arrow", "Left");
-    RegisterKey("Global", "RIGHT", "Right Arrow", "Right");
-    RegisterKey("Global", "SELECT", "Select", "Return,Enter,Space");
-    RegisterKey("Global", "ESCAPE", "Escape", "Esc");
-    RegisterKey("Global", "MENU", "Pop-up menu", "M");
-    RegisterKey("Global", "INFO", "More information", "I");
-
-    RegisterKey("Global", "PAGEUP", "Page Up", "PgUp");
-    RegisterKey("Global", "PAGEDOWN", "Page Down", "PgDown");
-
-    RegisterKey("Global", "PREVVIEW", "Previous View", "Home");
-    RegisterKey("Global", "NEXTVIEW", "Next View", "End");
-
-    RegisterKey("Global", "HELP", "Help", "F1");
-
-    RegisterKey("Global", "0", "0", "0");
-    RegisterKey("Global", "1", "1", "1");
-    RegisterKey("Global", "2", "2", "2");
-    RegisterKey("Global", "3", "3", "3");
-    RegisterKey("Global", "4", "4", "4");
-    RegisterKey("Global", "5", "5", "5");
-    RegisterKey("Global", "6", "6", "6");
-    RegisterKey("Global", "7", "7", "7");
-    RegisterKey("Global", "8", "8", "8");
-    RegisterKey("Global", "9", "9", "9");
-}
-
-MythMainWindow::~MythMainWindow()
-{
-    delete d;
-}
-
-void MythMainWindow::closeEvent(QCloseEvent *e)
-{
-    (void)e;
-    qApp->quit();
-}
-
-void MythMainWindow::Init(void)
-{
-    gContext->GetScreenSettings(d->xbase, d->screenwidth, d->wmult,
-                                d->ybase, d->screenheight, d->hmult);
-    setGeometry(d->xbase, d->ybase, d->screenwidth, d->screenheight);
-    setFixedSize(QSize(d->screenwidth, d->screenheight));
-
-    setFont(gContext->GetMediumFont());
-
-    bool hideCursor = gContext->GetNumSetting("HideMouseCursor", 1);
-    setCursor((hideCursor) ? (Qt::BlankCursor) : (Qt::ArrowCursor));
-#ifdef QWS
-#if QT_VERSION >= 0x030300
-    QWSServer::setCursorVisible(!hideCursor);
-#endif
-#endif
-
-    if (gContext->GetNumSetting("RunFrontendInWindow", 0))
-        d->does_fill_screen = false;
-    else
-        d->does_fill_screen = true;
-
-    // Set window border based on fullscreen attribute
-    Qt::WFlags flags = 0;
-    if (d->does_fill_screen)
-        flags = Qt::WStyle_Customize  |
-                Qt::WStyle_NoBorder;
-    else
-        flags = Qt::WStyle_Customize | Qt::WStyle_NormalBorder;
-
-    // Workarounds for Qt/Mac bugs
-#ifdef Q_WS_MACX
-    if (d->does_fill_screen)
-    {
-  #if QT_VERSION >= 0x030303
-        flags = Qt::WStyle_Customize | Qt::WStyle_Splash;
-  #else
-        // Qt/Mac 3.3.2 and earlier have problems with input focus
-        // when a borderless window is created more than once,
-        // so we have to force windows to have borders
-        flags = Qt::WStyle_Customize | Qt::WStyle_DialogBorder;
-
-        // Move this window up enough to hide its title bar, which in
-        // all the OS X releases so far is the same height as the menubar
-        d->ybase -= GetMBarHeight();
-  #endif
-    }
-#endif  
-
-    reparent(parentWidget(), flags, pos());
-
-    gContext->ThemeWidget(this);
-
-    Show();
-    move(d->xbase, d->ybase);
-}
-
-void MythMainWindow::Show(void)
-{
-    show();
-#ifdef Q_WS_MACX
-    if (d->does_fill_screen)
-        HideMenuBar();
-    else
-        ShowMenuBar();
-#endif
-
-    setActiveWindow();
-    raise();
-    qApp->wakeUpGuiThread();    // ensures that setActiveWindow() occurs
-}
-
-void MythMainWindow::attach(QWidget *child)
-{
-    if (currentWidget())
-        currentWidget()->setEnabled(false);
-
-    d->widgetList.push_back(child);
-    child->raise();
-    child->setFocus();
-}
-
-void MythMainWindow::detach(QWidget *child)
-{
-    if (d->widgetList.back() != child)
-    {
-        //cerr << "Not removing top-most widget\n";
-        return;
-    }
-
-    d->widgetList.pop_back();
-    QWidget *current = currentWidget();
-
-    if (current)
-    {
-        current->setEnabled(true);
-        current->setFocus();
-    }
-
-    if (d->exitingtomain)
-        QApplication::postEvent(this, new ExitToMainMenuEvent());
-}
-
-QWidget *MythMainWindow::currentWidget(void)
-{
-    if (d->widgetList.size() > 0)
-        return d->widgetList.back();
-    return NULL;
-}
-
-void MythMainWindow::ExitToMainMenu(void)
-{
-    QWidget *current = currentWidget();
-
-    if (current && d->exitingtomain)
-    {
-        if (current->name() != QString("mainmenu"))
-        {
-            if (current->name() == QString("video playback window"))
-            {
-                MythEvent *me = new MythEvent("EXIT_TO_MENU");
-                QApplication::postEvent(current, me);
-                d->exitingtomain = true;
-            }
-            else if (MythDialog *dial = dynamic_cast<MythDialog*>(current))
-            {
-                (void)dial;
-                QKeyEvent *key = new QKeyEvent(QEvent::KeyPress, d->escapekey, 
-                                               0, Qt::NoButton);
-                QObject *key_target = getTarget(*key);
-                QApplication::postEvent(key_target, key);
-                d->exitingtomain = true;
-            }
-        }
-        else
-        {
-            d->exitingtomain = false;
-            if (d->exitmenucallback)
-            {
-                void (*callback)(void) = d->exitmenucallback;
-                d->exitmenucallback = NULL;
-
-                callback();
-            }
-            else if (d->exitmenumediadevicecallback)
-            {
-                void (*callback)(MythMediaDevice*) = d->exitmenumediadevicecallback;
-                MythMediaDevice * mediadevice = d->mediadeviceforcallback;
-                d->mediadeviceforcallback = NULL;
-                callback(mediadevice);
-            }
-        }
-    }
-}
-
-bool MythMainWindow::TranslateKeyPress(const QString &context,
-                                       QKeyEvent *e, QStringList &actions)
-{
-    actions.clear();
-    int keynum = d->TranslateKeyNum(e);
-
-    if (d->jumpMap.count(keynum) > 0 && d->exitmenucallback == NULL)
-    {
-        d->exitingtomain = true;
-        d->exitmenucallback = d->jumpMap[keynum]->callback;
-        QApplication::postEvent(this, new ExitToMainMenuEvent());
-        return false;
-    }
-
-    bool retval = false;
-
-    if (d->keyContexts[context])
-    {
-        if (d->keyContexts[context]->GetMapping(keynum, actions))
-            retval = true;
-    }
-
-    if (d->keyContexts["Global"]->GetMapping(keynum, actions))
-        retval = true;
-
-    return retval;
-}
-
-void MythMainWindow::RegisterKey(const QString &context, const QString &action,
-                                 const QString &description, const QString &key)
-{
-    QString keybind = key;
-
-    MSqlQuery query(MSqlQuery::InitCon());
-    if (query.isConnected())
-    {
-        query.prepare("SELECT keylist FROM keybindings WHERE "
-                      "context = :CONTEXT AND action = :ACTION AND "
-                      "hostname = :HOSTNAME ;");
-        query.bindValue(":CONTEXT", context);
-        query.bindValue(":ACTION", action);
-        query.bindValue(":HOSTNAME", gContext->GetHostName());
-    
-        if (query.exec() && query.isActive() && query.size() > 0)
-        {
-            query.next();
-            keybind = query.value(0).toString();
-        }
-        else
-        {
-            QString inskey = keybind;
-            inskey.replace('\\', "\\\\");
-            inskey.replace('\"', "\\\"");
-    
-            query.prepare("INSERT INTO keybindings (context, action, "
-                          "description, keylist, hostname) VALUES "
-                          "( :CONTEXT, :ACTION, :DESCRIPTION, :KEYLIST, "
-                          ":HOSTNAME );");
-            query.bindValue(":CONTEXT", context);
-            query.bindValue(":ACTION", action);
-            query.bindValue(":DESCRIPTION", description);
-            query.bindValue(":KEYLIST", inskey);
-            query.bindValue(":HOSTNAME", gContext->GetHostName());
-
-            if (!query.exec() || !query.isActive())
-            {
-                MythContext::DBError("Insert Keybinding", query);
-            }
-        }
-    }
-
-
-    QKeySequence keyseq(keybind);
-
-    if (!d->keyContexts[context])
-        d->keyContexts.insert(context, new KeyContext());
-
-    for (unsigned int i = 0; i < keyseq.count(); i++)
-    {
-        int keynum = keyseq[i];
-        keynum &= ~Qt::UNICODE_ACCEL;
-
-        QStringList dummyaction = "";
-        if (d->keyContexts[context]->GetMapping(keynum, dummyaction))
-        {
-            VERBOSE(VB_GENERAL, QString("Key %1 is bound to multiple actions "
-                                        "in context %2.")
-                    .arg(keybind).arg(context));
-        }
-
-        d->keyContexts[context]->AddMapping(keynum, action);
-        //VERBOSE(VB_GENERAL, QString("Binding: %1 to action: %2 (%3)")
-        //                           .arg(key).arg(action)
-        //                           .arg(context));
-
-        if (action == "ESCAPE" && context == "Global" && i == 0)
-            d->escapekey = keynum;
-    }
-}
-
-void MythMainWindow::RegisterJump(const QString &destination, 
-                                  const QString &description,
-                                  const QString &key, void (*callback)(void))
-{
-    QString keybind = key;
-
-    MSqlQuery query(MSqlQuery::InitCon());
-    if (query.isConnected())
-    {
-        query.prepare("SELECT keylist FROM jumppoints WHERE "
-                      "destination = :DEST and hostname = :HOST ;");
-        query.bindValue(":DEST", destination);
-        query.bindValue(":HOST", gContext->GetHostName());
-    
-        if (query.exec() && query.isActive() && query.size() > 0)
-        {
-            query.next();
-            keybind = query.value(0).toString();
-        }
-        else
-        {
-            QString inskey = keybind;
-            inskey.replace('\\', "\\\\");
-            inskey.replace('\"', "\\\"");
-    
-            query.prepare("INSERT INTO jumppoints (destination, description, "
-                          "keylist, hostname) VALUES ( :DEST, :DESC, :KEYLIST, "
-                          ":HOST );");
-            query.bindValue(":DEST", destination);
-            query.bindValue(":DEDC", description);
-            query.bindValue(":KEYLIST", inskey);
-            query.bindValue(":HOST", gContext->GetHostName());
-
-            if (!query.exec() || !query.isActive())
-            { 
-                MythContext::DBError("Insert Jump Point", query);
-            } 
-        }
-    }
-
-
-    JumpData jd = { callback, destination, description };
-    d->destinationMap[destination] = jd;
- 
-    QKeySequence keyseq(keybind);
-
-    if (!keyseq.isEmpty())
-    {
-        int keynum = keyseq[0];
-        keynum &= ~Qt::UNICODE_ACCEL;
-
-        if (d->jumpMap.count(keynum) == 0)
-        {
-            //VERBOSE(VB_GENERAL, QString("Binding: %1 to JumpPoint: %2")
-            //                           .arg(keybind).arg(destination));
-
-            d->jumpMap[keynum] = &d->destinationMap[destination];
-        }
-        else
-        {
-            VERBOSE(VB_GENERAL, QString("Key %1 is already bound to a jump "
-                                        "point.").arg(keybind));
-        }
-    }
-    //else
-    //    VERBOSE(VB_GENERAL, QString("JumpPoint: %2 exists, no keybinding")
-    //                               .arg(destination));
-}
-
-void MythMainWindow::JumpTo(const QString& destination)
-{
-    if (d->destinationMap.count(destination) > 0 && d->exitmenucallback == NULL)
-    {
-        d->exitingtomain = true;
-        d->exitmenucallback = d->destinationMap[destination].callback;
-        QApplication::postEvent(this, new ExitToMainMenuEvent());
-        return;
-    }
-}
-
-bool MythMainWindow::DestinationExists(const QString& destination) const
-{
-    return (d->destinationMap.count(destination) > 0) ? true : false;
-}
-
-void MythMainWindow::RegisterMediaHandler(const QString &destination,
-                                          const QString &description,
-                                          const QString &key, 
-                              void (*callback)(MythMediaDevice*mediadevice),
-                                          int mediaType)
-{
-    (void)key;
-
-    if (d->mediaHandlerMap.count(destination) == 0) 
-    {
-        MHData mhd = { callback, mediaType, destination, description };
-
-        VERBOSE(VB_GENERAL, QString("Registering %1 as a media handler")
-                                   .arg(destination));
-
-        d->mediaHandlerMap[destination] = mhd;
-    }
-    else 
-    {
-       VERBOSE(VB_GENERAL, QString("%1 is already registered as a media "
-                                   "handler.").arg(destination));
-    }
-}
-
-void MythMainWindow::RegisterMediaPlugin(const QString &name, 
-                                         const QString &desc, 
-                                         MediaPlayCallback fn)
-{
-    if (d->mediaPluginMap.count(name) == 0) 
-    {
-        VERBOSE(VB_GENERAL, QString("Registering %1 as a media playback "
-                                    "plugin.").arg(name));
-        MPData mpd = {desc, fn};
-        d->mediaPluginMap[name] = mpd;
-    } 
-    else
-    {
-        VERBOSE(VB_GENERAL, QString("%1 is already registered as a media "
-                                    "playback plugin.").arg(name));
-    }
-}
-
-
-bool MythMainWindow::HandleMedia(QString& handler, const QString &mrl, const QString& plot,
-                                 const QString& title, const QString& director, int lenMins, const QString& year)
-{
-    if (handler.length() < 1)
-        handler = "Internal";
-
-    // Let's see if we have a plugin that matches the handler name...
-    if (d->mediaPluginMap.count(handler)) 
-    {
-        d->mediaPluginMap[handler].playFn(mrl, plot, title, director, lenMins, year);
-        return true;
-    }
-
-    return false;
-}
-
-void MythMainWindow::keyPressEvent(QKeyEvent *e)
-{
-    QWidget *current = currentWidget();
-    if (current && current->isEnabled())
-        qApp->notify(current, e);
-    else
-        QDialog::keyPressEvent(e);
-}
-
-void MythMainWindow::customEvent(QCustomEvent *ce)
-{
-    if (ce->type() == kExitToMainMenuEventType && d->exitingtomain)
-    {
-        ExitToMainMenu();
-    }
-    else if (ce->type() == kExternalKeycodeEventType)
-    {
-        ExternalKeycodeEvent *eke = (ExternalKeycodeEvent *)ce;
-        int keycode = eke->getKeycode();
-
-        QKeyEvent key(QEvent::KeyPress, keycode, 0, Qt::NoButton);
-
-        QObject *key_target = getTarget(key);
-
-        QApplication::sendEvent(key_target, &key);
-    }
-    else if (ce->type() == kMediaEventType) 
-    {
-        MediaEvent *media_event = (MediaEvent*)ce;
-        // Let's see which of our jump points are configured to handle this 
-        // type of media...  If there's more than one we'll want to show some 
-        // UI to allow the user to select which jump point to use. But for 
-        // now we're going to just use the first one.
-        QMap<QString, MHData>::Iterator itr = d->mediaHandlerMap.begin();
-        MythMediaDevice *pDev = media_event->getDevice();
-
-        if (pDev) 
-        {
-            QWidget * activewidget = qApp->focusWidget();
-            MythDialog * activedialog = NULL;
-            bool iscatched = false;
-            while (activewidget && !activedialog)
-            {
-                activedialog = dynamic_cast<MythDialog*>(activewidget);
-                if (!activedialog)
-                       activewidget = activewidget->parentWidget();
-            }
-            if (activedialog)
-                iscatched = activedialog->onMediaEvent(pDev);
-            if (!iscatched)
-            {
-                while (itr != d->mediaHandlerMap.end())
-                {
-                   if ((itr.data().MediaType & (int)pDev->getMediaType()))
-                   {
-                       VERBOSE(VB_ALL, "Found a handler");
-                       d->exitingtomain = true;
-                       d->exitmenumediadevicecallback = itr.data().callback;
-                       d->mediadeviceforcallback = pDev;
-                       QApplication::postEvent(this, new ExitToMainMenuEvent());
-                       break;
-                   }
-                   itr++;
-                }
-            }
-        }
-    }
-#ifdef USE_LIRC
-    else if (ce->type() == kLircKeycodeEventType && !d->ignore_lirc_keys) 
-    {
-        LircKeycodeEvent *lke = (LircKeycodeEvent *)ce;
-        int keycode = lke->getKeycode();
-
-        if (keycode) 
-        {
-            gContext->ResetScreensaver();
-
-            int mod = keycode & MODIFIER_MASK;
-            int k = keycode & ~MODIFIER_MASK; /* trim off the mod */
-            int ascii = 0;
-            QString text;
-
-            if (k & UNICODE_ACCEL)
-            {
-                QChar c(k & ~UNICODE_ACCEL);
-                ascii = c.latin1();
-                text = QString(c);
-            }
-
-            QKeyEvent key(lke->isKeyDown() ? QEvent::KeyPress :
-                          QEvent::KeyRelease, k, ascii, mod, text);
-
-            QObject *key_target = getTarget(key);
-
-            QApplication::sendEvent(key_target, &key);
-        }
-        else
-        {
-            cerr << "LircClient warning: attempt to convert '"
-                 << lke->getLircText() << "' to a key sequence failed. Fix"
-                                           " your key mappings.\n";
-        }
-    }
-    else if (ce->type() == kLircMuteEventType)
-    {
-        LircMuteEvent *lme = (LircMuteEvent *)ce;
-        d->ignore_lirc_keys = lme->eventsMuted();
-    }
-#endif
-#ifdef USE_JOYSTICK_MENU
-    else if (ce->type() == kJoystickKeycodeEventType && !d->ignore_joystick_keys) 
-    {
-        JoystickKeycodeEvent *jke = (JoystickKeycodeEvent *)ce;
-        int keycode = jke->getKeycode();
-
-        if (keycode) 
-        {
-            gContext->ResetScreensaver();
-
-            int mod = keycode & MODIFIER_MASK;
-            int k = keycode & ~MODIFIER_MASK; /* trim off the mod */
-            int ascii = 0;
-            QString text;
-
-            if (k & UNICODE_ACCEL)
-            {
-                QChar c(k & ~UNICODE_ACCEL);
-                ascii = c.latin1();
-                text = QString(c);
-            }
-
-            QKeyEvent key(jke->isKeyDown() ? QEvent::KeyPress :
-                          QEvent::KeyRelease, k, ascii, mod, text);
-
-            QObject *key_target = getTarget(key);
-
-            QApplication::sendEvent(key_target, &key);
-        }
-        else
-        {
-            cerr << "JoystickMenuClient warning: attempt to convert '"
-                 << jke->getJoystickMenuText() << "' to a key sequence failed. Fix"
-                                           " your key mappings.\n";
-        }
-    }
-    else if (ce->type() == kJoystickMuteEventType)
-    {
-        JoystickMenuMuteEvent *jme = (JoystickMenuMuteEvent *)ce;
-        d->ignore_joystick_keys = jme->eventsMuted();
-    }
-#endif
-    else if (ce->type() == ScreenSaverEvent::kScreenSaverEventType)
-    {
-        ScreenSaverEvent *sse = (ScreenSaverEvent *)ce;
-        switch (sse->getSSEventType())
-        {
-            case ScreenSaverEvent::ssetDisable:
-            {
-                gContext->DoDisableScreensaver();
-                break;
-            }
-            case ScreenSaverEvent::ssetRestore:
-            {
-                gContext->DoRestoreScreensaver();
-                break;
-            }
-            case ScreenSaverEvent::ssetReset:
-            {
-                gContext->DoResetScreensaver();
-                break;
-            }
-            default:
-            {
-                cerr << "Unknown ScreenSaverEvent type: " <<
-                        sse->getSSEventType() << std::endl;
-            }
-        }
-    }
-}
-
-QObject *MythMainWindow::getTarget(QKeyEvent &key)
-{
-    QObject *key_target = NULL;
-
-    key_target = QWidget::keyboardGrabber();
-
-    if (!key_target)
-    {
-        QWidget *focus_widget = qApp->focusWidget();
-        if (focus_widget && focus_widget->isEnabled())
-        {
-            key_target = focus_widget;
-
-            // Yes this is special code for handling the
-            // the escape key.
-            if (key.key() == d->escapekey && focus_widget->topLevelWidget())
-                key_target = focus_widget->topLevelWidget();
-        }
-    }
-
-    if (!key_target)
-        key_target = this;
-
-    return key_target;
-}
-
 MythDialog::MythDialog(MythMainWindow *parent, const char *name, bool setsize)
           : QFrame(parent, name)
 {
@@ -965,9 +155,6 @@ void MythDialog::hide(void)
 
 void MythDialog::keyPressEvent( QKeyEvent *e )
 {
-    if (e->state() != 0)
-        return;
-
     bool handled = false;
     QStringList actions;
 
@@ -1270,6 +457,17 @@ int MythPopupBox::ExecPopup(QObject *target, const char *slot)
     return exec();
 }
 
+int MythPopupBox::ExecPopupAtXY(int destx, int desty,
+                            QObject *target, const char *slot)
+{
+    if (!target)
+        ShowPopupAtXY(destx, desty, this, SLOT(defaultExitHandler()));
+    else
+        ShowPopupAtXY(destx, desty, target, slot);
+
+    return exec();
+}
+
 void MythPopupBox::defaultButtonPressedHandler(void)
 {
     const QObjectList *objlist = children();
@@ -1423,9 +621,7 @@ MythProgressDialog::MythProgressDialog(const QString &message, int totalSteps)
     progress->setBackgroundOrigin(ParentOrigin);
     progress->setProgress(0);
 
-    steps = totalSteps / 1000;
-    if (steps == 0)
-        steps = 1;
+    setTotalSteps(totalSteps);
 
     if (class LCD * lcddev = LCD::Get())
     {
@@ -1460,12 +656,16 @@ void MythProgressDialog::Close(void)
 
 void MythProgressDialog::setProgress(int curprogress)
 {
-    float fProgress = (float)curprogress / (steps * 1000.0);
-    if (class LCD * lcddev = LCD::Get())
-        lcddev->setGenericProgress(fProgress);
     progress->setProgress(curprogress);
     if (curprogress % steps == 0)
+    {
         qApp->processEvents();
+        if (LCD * lcddev = LCD::Get())
+        {
+            float fProgress = (float)curprogress / m_totalSteps;
+            lcddev->setGenericProgress(fProgress);
+        }
+    }
 }
 
 void MythProgressDialog::keyPressEvent(QKeyEvent *e)
@@ -1486,30 +686,113 @@ void MythProgressDialog::keyPressEvent(QKeyEvent *e)
         MythDialog::keyPressEvent(e);
 }
 
+void MythProgressDialog::setTotalSteps(int totalSteps)
+{
+    m_totalSteps = totalSteps;
+    steps = totalSteps / 1000;
+    if (steps == 0)
+        steps = 1;
+}
+
+MythBusyDialog::MythBusyDialog(const QString &title)
+    : MythProgressDialog(title, 0), timer(NULL)
+{
+}
+
+MythBusyDialog::~MythBusyDialog()
+{
+    if (timer)
+    {
+        timer->disconnect();
+        timer->deleteLater();
+        timer = NULL;
+    }
+}
+
+void MythBusyDialog::start(int interval)
+{
+    if (!timer)
+        timer = new QTimer (this);
+
+    connect(timer, SIGNAL(timeout()),
+            this,  SLOT  (timeout()));
+
+    timer->start(interval);
+}
+
+void MythBusyDialog::Close(void)
+{
+    if (timer)
+    {
+        timer->disconnect();
+        timer->deleteLater();
+        timer = NULL;
+    }
+
+    MythProgressDialog::Close();
+}
+
+void MythBusyDialog::setProgress(void)
+{
+    progress->setProgress(progress->progress () + 10);
+    qApp->processEvents();
+    if (LCD *lcddev = LCD::Get())
+        lcddev->setGenericBusy();
+}
+
+void MythBusyDialog::timeout(void)
+{
+    setProgress();
+}
+
 MythThemedDialog::MythThemedDialog(MythMainWindow *parent, QString window_name,
                                    QString theme_filename, const char* name,
                                    bool setsize)
                 : MythDialog(parent, name, setsize)
 {
     setNoErase();
+
+    theme = NULL;
+
+    if (!loadThemedWindow(window_name, theme_filename))
+    {
+        QString msg = 
+            QString(tr("Could not locate '%1' in theme '%2'."
+                       "\n\nReturning to the previous menu."))
+            .arg(window_name).arg(theme_filename);
+        MythPopupBox::showOkPopup(gContext->GetMainWindow(),
+                                  tr("Missing UI Element"), msg);
+        done(-1);
+        return;
+    }
+}
+
+MythThemedDialog::MythThemedDialog(MythMainWindow *parent, const char* name,
+                                   bool setsize)
+                : MythDialog(parent, name, setsize)
+{
+    setNoErase();
+    theme = NULL;
+}
+
+bool MythThemedDialog::loadThemedWindow(QString window_name, QString theme_filename)
+{
+
+    if (theme)
+        delete theme;
+
     context = -1;
     my_containers.clear();
     widget_with_current_focus = NULL;
 
     redrawRect = QRect(0, 0, 0, 0);
 
-    //
-    //  Load the theme. Crap out if we can't find it.
-    //
-
     theme = new XMLParse();
     theme->SetWMult(wmult);
     theme->SetHMult(hmult);
     if (!theme->LoadTheme(xmldata, window_name, theme_filename))
     {
-        cerr << "dialogbox.o: Couldn't find your theme. I'm outta here" << endl;
-        cerr << window_name << " - " <<  theme_filename << endl;
-        exit(-31);
+        return false;
     }
 
     loadWindow(xmldata);
@@ -1543,6 +826,8 @@ MythThemedDialog::MythThemedDialog(MythMainWindow *parent, QString window_name,
 
     updateBackground();
     initForeground();
+
+    return true;
 }
 
 bool MythThemedDialog::buildFocusList()
@@ -1565,9 +850,11 @@ bool MythThemedDialog::buildFocusList()
         for (; i != all_ui_type_objects->end(); i++)
         {
             UIType *type = (*i);
-            if (type->canTakeFocus())
+            if (type->canTakeFocus() && !type->isHidden())
             {
-                focus_taking_widgets.append(type);
+                if (context == -1 || type->GetContext() == -1 || 
+                        context == type->GetContext())
+                    focus_taking_widgets.append(type);
             }
         }
         ++another_it;
@@ -1581,7 +868,8 @@ bool MythThemedDialog::buildFocusList()
 
 MythThemedDialog::~MythThemedDialog()
 {
-    delete theme;
+    if (theme)
+        delete theme;
 }
 
 void MythThemedDialog::loadWindow(QDomElement &element)
@@ -1610,9 +898,10 @@ void MythThemedDialog::loadWindow(QDomElement &element)
             }
             else
             {
-                cerr << "dialogbox.o: I don't understand this DOM Element:" 
-                     << e.tagName() << endl;
-                exit(-32);
+                VERBOSE(VB_IMPORTANT,
+                        QString("MythThemedDialog::loadWindow(): Do not "
+                                "understand DOM Element: '%1'. Ignoring.")
+                        .arg(e.tagName()));
             }
         }
     }
@@ -1632,9 +921,8 @@ void MythThemedDialog::parseContainer(QDomElement &element)
     theme->parseContainer(element, name, a_context, area);
     if (name.length() < 1)
     {
-        cerr << "dialogbox.o: I told an object to parse a container and it "
-                "didn't give me a name back\n";
-        exit(-33);
+        VERBOSE(VB_IMPORTANT, "Failed to parse a container. Ignoring.");
+        return;
     }
 
     LayerSet *container_reference = theme->GetSet(name);
@@ -1891,6 +1179,7 @@ bool MythThemedDialog::assignFirstFocus()
             widget_with_current_focus->takeFocus();
             return true;
         }
+        ++an_it;
     }
 
     return false;
@@ -2060,6 +1349,27 @@ UITextType* MythThemedDialog::getUITextType(const QString &name)
     return NULL;
 }
 
+UIRichTextType* MythThemedDialog::getUIRichTextType(const QString &name)
+{
+    QPtrListIterator<LayerSet> an_it(my_containers);
+    LayerSet *looper;
+
+    while( (looper = an_it.current()) != 0)
+    {
+        UIType *hunter = looper->GetType(name);
+        if (hunter)
+        {
+            UIRichTextType *hunted;
+            if ( (hunted = dynamic_cast<UIRichTextType*>(hunter)) )
+            {
+                return hunted;
+            }
+        }
+        ++an_it;
+    }
+    return NULL;
+}
+
 UIRemoteEditType* MythThemedDialog::getUIRemoteEditType(const QString &name)
 {
     QPtrListIterator<LayerSet> an_it(my_containers);
@@ -2312,6 +1622,27 @@ UIListTreeType* MythThemedDialog::getUIListTreeType(const QString &name)
     return NULL;
 }
 
+UIKeyboardType *MythThemedDialog::getUIKeyboardType(const QString &name)
+{
+    QPtrListIterator<LayerSet> an_it(my_containers);
+    LayerSet *looper;
+
+    while( (looper = an_it.current()) != 0)
+    {
+        UIType *hunter = looper->GetType(name);
+        if (hunter)
+        {
+            UIKeyboardType *hunted;
+            if ( (hunted = dynamic_cast<UIKeyboardType*>(hunter)) )
+            {
+                return hunted;
+            }
+        }
+        ++an_it;
+    }
+    return NULL;
+}
+
 LayerSet* MythThemedDialog::getContainer(const QString& name)
 {
     QPtrListIterator<LayerSet> an_it(my_containers);
@@ -2328,6 +1659,15 @@ LayerSet* MythThemedDialog::getContainer(const QString& name)
     return NULL;
 }
 
+fontProp* MythThemedDialog::getFont(const QString &name)
+{
+    fontProp* font = NULL;
+    if (theme)
+        font = theme->GetFont(name, true);
+
+    return font;
+}
+
 /*
 ---------------------------------------------------------------------
 */
@@ -2362,6 +1702,7 @@ MythPasswordDialog::MythPasswordDialog(QString message,
     password_editor->setEchoMode(QLineEdit::Password);
     password_editor->setGeometry(textWidth + 20,10,135,30);
     password_editor->setBackgroundOrigin(ParentOrigin);
+    password_editor->setAllowVirtualKeyboard(false);
     connect(password_editor, SIGNAL(textChanged(const QString &)),
             this, SLOT(checkPassword(const QString &)));
 
@@ -2414,12 +1755,13 @@ MythSearchDialog::MythSearchDialog(MythMainWindow *parent, const char *name)
 {
     // create the widgets
     caption = addLabel(QString(""));
-    
+
     editor = new MythRemoteLineEdit(this);
     connect(editor, SIGNAL(textChanged()), this, SLOT(searchTextChanged()));
     addWidget(editor);
-    editor->setFocus(); 
-    
+    editor->setFocus();
+    editor->setPopupPosition(VK_POSBOTTOMDIALOG); 
+
     listbox = new MythListBox(this);
     listbox->setScrollBar(false);
     listbox->setBottomScrollBar(false);
@@ -2528,6 +1870,16 @@ MythSearchDialog::~MythSearchDialog()
 ---------------------------------------------------------------------
  */
 
+/** \fn MythImageFileDialog::MythImageFileDialog(QString*,QString,MythMainWindow*,QString,QString,const char*,bool)
+ *  \brief Locates an image file dialog in the current theme.
+ *
+ *   MythImageFileDialog's expect there to be certain
+ *   elements in the theme, or they will fail.
+ *
+ *   For example, we use the size of the background
+ *   pixmap to define the geometry of the dialog. If
+ *   the pixmap ain't there, we need to fail.
+ */
 MythImageFileDialog::MythImageFileDialog(QString *result,
                                          QString top_directory,
                                          MythMainWindow *parent, 
@@ -2544,18 +1896,6 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     selected_file = result;
     initial_node = NULL;
     
-    //
-    //  MythImageFileDialog's expect there to be certain
-    //  elements in the theme, or they will fail.
-    //
-
-    //
-    //  For example, we use the size of the background
-    //  pixmap to define the geometry of the dialog. If
-    //  the pixmap ain't there, we need to fail.
-    //
-    
-
     UIImageType *file_browser_background = getUIImageType("file_browser_background");
     if (file_browser_background)
     {
@@ -2567,8 +1907,14 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     }
     else
     {
-        cerr << "myhdialogs.o: Could not find a UIImageType called file_browser_background in your theme." << endl;
-        exit(-34);
+        QString msg = 
+            QString(tr("The theme you are using is missing the "
+                       "'file_browser_background' "
+                       "element. \n\nReturning to the previous menu."));
+        MythPopupBox::showOkPopup(gContext->GetMainWindow(),
+                                  tr("Missing UI Element"), msg);
+        done(-1);
+        return;
     }
 
     //
@@ -2595,9 +1941,15 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
                 this, SLOT(handleTreeListEntered(int, IntVector*)));
     }
     else
-    {
-        cerr << "mythdialogs.o: Could not find a UIManagedTreeListType called file_browser in your theme." << endl;
-        exit(-35);
+    { 
+        QString msg = 
+            QString(tr("The theme you are using is missing the "
+                       "'file_browser' element. "
+                       "\n\nReturning to the previous menu."));
+        MythPopupBox::showOkPopup(gContext->GetMainWindow(),
+                                  tr("Missing UI Element"), msg);
+        done(-1);
+        return;
     }    
     
     //
@@ -2824,10 +2176,13 @@ MythScrollDialog::MythScrollDialog(MythMainWindow *parent,
                                    const char *name)
     : QScrollView(parent, name)
 {
-    if (!parent) {
-        std::cerr << "MythScrollDialog: Trying to create a dialog without a parent"
-                  << std::endl;
-        exit(-36);
+    if (!parent)
+    {
+        VERBOSE(VB_IMPORTANT, 
+                "MythScrollDialog: Programmer error, trying to create "
+                "a dialog without a parent.");
+        done(-1);
+        return;
     }
 
     m_parent     = parent;
@@ -2916,9 +2271,12 @@ void MythScrollDialog::setArea(int w, int h)
 void MythScrollDialog::setAreaMultiplied(int areaWTimes, int areaHTimes)
 {
     if (areaWTimes < 1 || areaHTimes < 1) {
-        std::cerr << "MythScrollDialog: invalid areaWTimes or areaHTimes"
-                  << std::endl;
-        exit(-37);
+        VERBOSE(VB_IMPORTANT,
+                QString("MythScrollDialog::setAreaMultiplied(%1,%2): "
+                        "Warning, Invalid areaWTimes or areaHTimes. "
+                        "Setting to 1.")
+                .arg(areaWTimes).arg(areaHTimes));
+        areaWTimes = areaHTimes = 1;
     }
 
     resizeContents(m_screenWidth*areaWTimes,
@@ -2994,9 +2352,6 @@ void MythScrollDialog::setResult(int r)
 
 void MythScrollDialog::keyPressEvent(QKeyEvent *e)
 {
-    if (e->state() != 0)
-        return;
-
     bool handled = false;
     QStringList actions;
 
