@@ -19,10 +19,15 @@
 #include <qradiobutton.h>
 #include <qimage.h>
 #include <qlabel.h>
-#include <qtimer.h>
+#include <qtimer.h> 
 
 #include <vector>
 
+using namespace std;
+
+#include "virtualkeyboard.h"
+
+
 // These widgets follow these general navigation rules:
 //
 // - Up and Down shift focus to the previous/next widget in the tab
@@ -117,15 +122,23 @@ class MythLineEdit : public QLineEdit
     Q_OBJECT
   public:
     MythLineEdit(QWidget *parent=NULL, const char* widgetName=0) :
-      QLineEdit(parent, widgetName) { rw = true; };
+      QLineEdit(parent, widgetName)
+	    { rw = true; allowVirtualKeyboard = true; Init(); };
 
     MythLineEdit(const QString& contents, QWidget *parent=NULL, 
                  const char* widgetName=0) :
-      QLineEdit(contents, parent, widgetName) { rw = true; };
+      QLineEdit(contents, parent, widgetName)
+	    { rw = true; allowVirtualKeyboard = true; Init(); };
+
+    virtual ~MythLineEdit();
 
     void setHelpText(QString help) { helptext = help; };
     void setRW(bool readwrite = true) { rw = readwrite; };
     void setRO() { rw = false; };
+    void setAllowVirtualKeyboard(bool allowKbd = true)
+	       { allowVirtualKeyboard = allowKbd; }
+    void setPopupPosition(PopupPosition pos) { popupPosition = pos; }
+    PopupPosition getPopupPosition(void) { return popupPosition; }
 
  public slots:
      virtual void setText(const QString& text);
@@ -137,10 +150,16 @@ class MythLineEdit : public QLineEdit
     virtual void keyPressEvent(QKeyEvent *e);
     virtual void focusInEvent(QFocusEvent *e); 
     virtual void focusOutEvent(QFocusEvent *e);
+    virtual void hideEvent(QHideEvent *e);
+    virtual void mouseDoubleClickEvent(QMouseEvent *e);
+    void Init(void);
 
   private:
+    VirtualKeyboard *popup;
     QString helptext;
     bool rw;
+    bool allowVirtualKeyboard;
+    PopupPosition popupPosition;
 };
 
 // A Line edit that does special things when you press number keys 
@@ -148,7 +167,7 @@ class MythRemoteLineEdit : public QTextEdit
 {
     Q_OBJECT
   public:
-    
+
     MythRemoteLineEdit( QWidget * parent, const char * name = 0 );
     MythRemoteLineEdit( const QString & contents, QWidget * parent, const char * name = 0 );    
     MythRemoteLineEdit( QFont *a_font, QWidget * parent, const char * name = 0 );    
@@ -157,9 +176,14 @@ class MythRemoteLineEdit : public QTextEdit
     void setHelpText(QString help) { helptext = help; }
     void setCycleTime(float desired_interval); // in seconds
     void setCharacterColors(QColor unselected, QColor selected, QColor special);
+    void insert(QString text);
+    void backspace();
+    void del();
+    void setPopupPosition(PopupPosition pos) { popupPosition = pos; };
+    PopupPosition getPopupPosition(void) { return popupPosition; };
 
   signals:
-    
+
     void    shiftState(bool);
     void    cycleState(QString current_choice, QString set);
     void    changeHelpText(QString);
@@ -208,12 +232,15 @@ class MythRemoteLineEdit : public QTextEdit
     QColor  col_unselected;
     QColor  col_selected;
     QColor  col_special;
-        
+
     QString  hex_unselected;
     QString  hex_selected;
     QString  hex_special;
 
     int m_lines;
+
+    VirtualKeyboard *popup;
+    PopupPosition popupPosition;
 };
 
 class MythTable : public QTable
@@ -260,6 +287,8 @@ class MythPushButton : public QPushButton
     QColor origcolor;
     QString helptext;
     bool arrowAccel;
+
+    QStringList keyPressActions;
 };
 
 class MythCheckBox: public QCheckBox
@@ -338,6 +367,8 @@ class MythListBox: public QListBox {
     void changeHelpText(QString);
     void accepted(int);
     void menuButtonPressed(int);
+    void editButtonPressed(int);
+    void deleteButtonPressed(int);
 
   private:
     QString helptext;
