@@ -1,156 +1,27 @@
 #include "osdsurface.h"
 #include "dithertable.h"
+#include "mythcontext.h"
 
-#include <iostream>
+#include <algorithm>
 using namespace std;
 
-#define MM_MMX    0x0001 /* standard MMX */
-#define MM_3DNOW  0x0004 /* AMD 3DNOW */
-#define MM_MMXEXT 0x0002 /* SSE integer functions or AMD MMX ext */
-#define MM_SSE    0x0008 /* SSE functions */
-#define MM_SSE2   0x0010 /* PIV SSE2 functions */
+#ifdef MMX
 
-#ifdef i386
-
-#include "mmx.h"
-
-#define cpuid(index,eax,ebx,ecx,edx)\
-    __asm __volatile\
-        ("movl %%ebx, %%esi\n\t"\
-         "cpuid\n\t"\
-         "xchgl %%ebx, %%esi"\
-         : "=a" (eax), "=S" (ebx),\
-           "=c" (ecx), "=d" (edx)\
-         : "0" (index));
-
-#define        emms()                  __asm__ __volatile__ ("emms")
-
-/* Function to test if multimedia instructions are supported...  */
-int mm_support(void)
-{
-    int rval;
-    int eax, ebx, ecx, edx;
-
-    __asm__ __volatile__ (
-                          /* See if CPUID instruction is supported ... */
-                          /* ... Get copies of EFLAGS into eax and ecx */
-                          "pushf\n\t"
-                          "popl %0\n\t"
-                          "movl %0, %1\n\t"
-
-                          /* ... Toggle the ID bit in one copy and store */
-                         /*     to the EFLAGS reg */
-                          "xorl $0x200000, %0\n\t"
-                          "push %0\n\t"
-                          "popf\n\t"
-
-                          /* ... Get the (hopefully modified) EFLAGS */
-                          "pushf\n\t"
-                          "popl %0\n\t"
-                          : "=a" (eax), "=c" (ecx)
-                          :
-                          : "cc"
-                          );
-
-    if (eax == ecx)
-        return 0; /* CPUID not supported */
-
-    cpuid(0, eax, ebx, ecx, edx);
-
-    if (ebx == 0x756e6547 &&
-        edx == 0x49656e69 &&
-        ecx == 0x6c65746e) {
-
-        /* intel */
-    inteltest:
-        cpuid(1, eax, ebx, ecx, edx);
-        if ((edx & 0x00800000) == 0)
-            return 0;
-        rval = MM_MMX;
-        if (edx & 0x02000000)
-            rval |= MM_MMXEXT | MM_SSE;
-        if (edx & 0x04000000)
-            rval |= MM_SSE2;
-        return rval;
-    } else if (ebx == 0x68747541 &&
-               edx == 0x69746e65 &&
-               ecx == 0x444d4163) {
-        /* AMD */
-        cpuid(0x80000000, eax, ebx, ecx, edx);
-        if ((unsigned)eax < 0x80000001)
-            goto inteltest;
-        cpuid(0x80000001, eax, ebx, ecx, edx);
-        if ((edx & 0x00800000) == 0)
-            return 0;
-        rval = MM_MMX;
-        if (edx & 0x80000000)
-            rval |= MM_3DNOW;
-        if (edx & 0x00400000)
-            rval |= MM_MMXEXT;
-        return rval;
-    } else if (ebx == 0x746e6543 &&
-               edx == 0x48727561 &&
-               ecx == 0x736c7561) {  /*  "CentaurHauls" */
-        /* VIA C3 */
-        cpuid(0x80000000, eax, ebx, ecx, edx);
-        if ((unsigned)eax < 0x80000001)
-            goto inteltest;
-        cpuid(0x80000001, eax, ebx, ecx, edx);
-        rval = 0;
-        if ( edx & ( 1 << 31) )
-          rval |= MM_3DNOW;
-        if ( edx & ( 1 << 23) )
-          rval |= MM_MMX;
-        if ( edx & ( 1 << 24) )
-          rval |= MM_MMXEXT;
-        if (rval==0)
-            goto inteltest;
-        return rval;
-    } else if (ebx == 0x69727943 &&
-               edx == 0x736e4978 &&
-               ecx == 0x64616574) {
-        /* Cyrix Section */
-        /* See if extended CPUID level 80000001 is supported */
-        /* The value of CPUID/80000001 for the 6x86MX is undefined
-           according to the Cyrix CPU Detection Guide (Preliminary
-           Rev. 1.01 table 1), so we'll check the value of eax for
-           CPUID/0 to see if standard CPUID level 2 is supported.
-           According to the table, the only CPU which supports level
-           2 is also the only one which supports extended CPUID levels.
-        */
-        if (eax != 2)
-            goto inteltest;
-        cpuid(0x80000001, eax, ebx, ecx, edx);
-        if ((eax & 0x00800000) == 0)
-            return 0;
-        rval = MM_MMX;
-        if (eax & 0x01000000)
-            rval |= MM_MMXEXT;
-        return rval;
-    } else if (ebx == 0x756e6547 &&
-               edx == 0x54656e69 &&
-               ecx == 0x3638784d) {
-        /* Tranmeta Crusoe */
-        cpuid(0x80000000, eax, ebx, ecx, edx);
-        if ((unsigned)eax < 0x80000001)
-            return 0;
-        cpuid(0x80000001, eax, ebx, ecx, edx);
-        if ((edx & 0x00800000) == 0)
-            return 0;
-        return MM_MMX;
-    } else {
-        return 0;
-    }
+extern "C" {
+#include "config.h"
+#include "dsputil.h"
+#include "i386/mmx.h"
 }
+
 #endif
 
 OSDSurface::OSDSurface(int w, int h)
 {
-    yuvbuffer = new unsigned char[w * h * 3 / 2];
+    yuvbuffer = new unsigned char[w * (h + 2) * 3 / 2];
     y = yuvbuffer;
     u = yuvbuffer + w * h;
     v = u + w * h / 4;
-    alpha = new unsigned char[w * h];
+    alpha = new unsigned char[w * (h + 2)];
 
     width = w;
     height = h;
@@ -188,7 +59,7 @@ OSDSurface::OSDSurface(int w, int h)
     blendcolumnfunc = &blendcolumn;
     blendcolorfunc = &blendcolor;
     blendconstfunc = &blendconst;
-#if defined(i386) && defined(MMX)
+#if defined(MMX)
     usemmx = (mm_support() & MM_MMX);
     if (usemmx)
     {
@@ -202,6 +73,7 @@ OSDSurface::OSDSurface(int w, int h)
         blendconstfunc = &blendconst_mmx;
     }
 #endif
+    revision = 0;
 }
 
 OSDSurface::~OSDSurface()
@@ -212,6 +84,7 @@ OSDSurface::~OSDSurface()
 
 void OSDSurface::Clear(void)
 {
+    QMutexLocker lock(&usedRegionsLock);
     memset(y, 0, size);
     memset(u, 127, size / 4);
     memset(v, 127, size / 4);
@@ -221,6 +94,7 @@ void OSDSurface::Clear(void)
 
 void OSDSurface::ClearUsed(void)
 {
+    QMutexLocker lock(&usedRegionsLock);
     QMemArray<QRect> rects = usedRegions.rects();
     QMemArray<QRect>::Iterator it = rects.begin();
     QRect drawRect;
@@ -239,6 +113,11 @@ void OSDSurface::ClearUsed(void)
         cwidth = drawRect.width();
         uvcwidth = cwidth / 2;
 
+        if (startline < 0) startline = 0;
+        if (endline >= height) endline = height - 1;
+        if (startcol < 0) endcol = 0;
+        if (endcol >= width) endcol = width - 1;
+
         for (int line = startline; line <= endline; line++)
         {
             yoffset = line * width;
@@ -258,6 +137,7 @@ void OSDSurface::ClearUsed(void)
 
 bool OSDSurface::IntersectsDrawn(QRect &newrect)
 {
+    QMutexLocker lock(&usedRegionsLock);
     QMemArray<QRect> rects = usedRegions.rects();
     QMemArray<QRect>::Iterator it = rects.begin();
     for (; it != rects.end(); ++it)
@@ -268,6 +148,7 @@ bool OSDSurface::IntersectsDrawn(QRect &newrect)
 
 void OSDSurface::AddRect(QRect &newrect)
 {
+    QMutexLocker lock(&usedRegionsLock);
     usedRegions = usedRegions.unite(newrect);
 }
 
@@ -284,7 +165,7 @@ static inline void blendalpha8_c(unsigned char *src, unsigned char *dest,
          dest[i] = blendColorsAlpha(src[i], dest[i], alpha[i * mult]);
 }
 
-#if i386
+#ifdef MMX
 // cribbed from Jason Tackaberry's vf_bmovl2 source
 static inline void blendalpha8_mmx(unsigned char *src, unsigned char *dest,
                                    unsigned char *alpha, bool uvplane)
@@ -378,22 +259,23 @@ static inline void blendalpha8_mmx(unsigned char *src, unsigned char *dest,
 }
 #endif
 
-blendtoyv12_8_fun blendtoyv12_8_init(OSDSurface *surface)
+blendtoyv12_8_fun blendtoyv12_8_init(const OSDSurface *surface)
 {
-#if i386
+    (void)surface;
+#ifdef MMX
     if (surface->usemmx)
         return blendalpha8_mmx;
 #endif
     return blendalpha8_c;
 }
 
-static inline void blendtoargb_8_c(OSDSurface *surf, unsigned char *src,
+static inline void blendtoargb_8_c(const OSDSurface *surf, unsigned char *src,
                                    unsigned char *usrc, unsigned char *vsrc,
                                    unsigned char *alpha, unsigned char *dest)
 {
     int r, g, b, y0;
     int cb, cr, r_add, g_add, b_add;
-    unsigned char *cm = surf->cm;
+    const unsigned char *cm = surf->cm;
 
     int i, j;
 
@@ -407,9 +289,9 @@ static inline void blendtoargb_8_c(OSDSurface *surf, unsigned char *src,
     }
 }
 
-#if i386
+#ifdef MMX
 #define movntq(src, dest) movq_r2m(src, dest);
-static inline void blendtoargb_8_mmx(OSDSurface * /*surf*/, unsigned char *src,
+static inline void blendtoargb_8_mmx(const OSDSurface * /*surf*/, unsigned char *src,
                                      unsigned char *usrc, unsigned char *vsrc,
                                      unsigned char *alpha, unsigned char *dest)
 {
@@ -504,9 +386,10 @@ static inline void blendtoargb_8_mmx(OSDSurface * /*surf*/, unsigned char *src,
 }
 #endif
 
-blendtoargb_8_fun blendtoargb_8_init(OSDSurface *surface)
+blendtoargb_8_fun blendtoargb_8_init(const OSDSurface *surface)
 {
-#if i386
+    (void)surface;
+#ifdef MMX
     if (surface->usemmx)
         return blendtoargb_8_mmx;
 #endif
@@ -519,7 +402,7 @@ struct dither8_context
     int amask;
     int ishift;
     int imask;
-#if i386
+#ifdef MMX
     mmx_t amask_mmx;
     mmx_t imask_mmx;
 #endif
@@ -541,7 +424,7 @@ static inline void dithertoia44_8_c(unsigned char *src, unsigned char *dest,
     }
 }
 
-#if i386
+#ifdef MMX
 static inline void dithertoia44_8_mmx(unsigned char *src, unsigned char *dest,
                                       unsigned char *alpha,
                                       const unsigned char *dmp, int xpos,
@@ -619,9 +502,9 @@ static inline void dithertoia44_8_mmx(unsigned char *src, unsigned char *dest,
 }
 #endif
 
-dithertoia44_8_fun dithertoia44_8_init(OSDSurface* /*surface*/)
+dithertoia44_8_fun dithertoia44_8_init(const OSDSurface* /*surface*/)
 {
-#if i386
+#ifdef MMX
 // mmx version seems to be about the same speed, no reason to use it.
 //    if (surface->usemmx)
 //        return dithertoia44_8_mmx;
@@ -629,7 +512,7 @@ dithertoia44_8_fun dithertoia44_8_init(OSDSurface* /*surface*/)
     return dithertoia44_8_c;
 }
 
-#if i386
+#ifdef MMX
 static mmx_t mask_0f = {0x0f0f0f0f0f0f0f0fLL};
 static mmx_t mask_f0 = {0xf0f0f0f0f0f0f0f0LL};
 #endif
@@ -643,7 +526,7 @@ dither8_context *init_dithertoia44_8_context(bool first)
         context->amask = 0x0f;
         context->ishift = 4;
         context->imask = 0xf0;
-#if i386
+#ifdef MMX
         context->amask_mmx = mask_0f;
         context->imask_mmx = mask_f0;
 #endif
@@ -654,7 +537,7 @@ dither8_context *init_dithertoia44_8_context(bool first)
         context->amask = 0xf0;
         context->ishift = 0;
         context->imask = 0x0f;
-#if i386
+#ifdef MMX
         context->amask_mmx = mask_f0;
         context->imask_mmx = mask_0f;
 #endif
@@ -668,4 +551,339 @@ void delete_dithertoia44_8_context(dither8_context *context)
     delete context;
 }
 
+/** \fn OSDSurface::BlendToYV12(unsigned char*,unsigned char*,unsigned char*,int,int,int) const
+ *  \brief Alpha blends OSDSurface to yuv buffer of the same size.
+ *  \param yptrdest Pointer to Y buffer to blend OSD to.
+ *  \param uptrdest Pointer to U buffer to blend OSD to.
+ *  \param vptrdest Pointer to V buffer to blend OSD to.
+ */
+void OSDSurface::BlendToYV12(unsigned char *yptrdest,
+                             unsigned char *uptrdest,
+                             unsigned char *vptrdest,
+                             int ystride, int ustride, int vstride) const
+{
+    QMutexLocker lock(&usedRegionsLock);
+    const OSDSurface *surface = this;
+    blendtoyv12_8_fun blender = blendtoyv12_8_init(surface);
 
+    QMemArray<QRect> rects = surface->usedRegions.rects();
+    QMemArray<QRect>::Iterator it = rects.begin();
+    for (; it != rects.end(); ++it)
+    {
+        int begx = max((*it).left(),   0);
+        int begy = max((*it).top(),    0);
+        int endx = min((*it).right(),  width  - 1);
+        int endy = min((*it).bottom(), height - 1);
+
+        for (int y = begy; y <= endy; y++)
+        {
+            int ysrcoff = y * surface->width;
+            int ydstoff = y * ystride;
+
+            unsigned char *src   = surface->y     + ysrcoff + begx;
+            unsigned char *alpha = surface->alpha + ysrcoff + begx;
+            unsigned char *dest  = yptrdest       + ydstoff + begx;
+
+            for (int x = begx; x <= endx;)
+            {
+                if (x + 8 >= endx)
+                {
+                    if (*alpha != 0)
+                        *dest = blendColorsAlpha(*src, *dest, *alpha);
+                    src++;
+                    dest++;
+                    alpha++;
+                    x++;
+                }
+                else
+                {
+                    blender(src, dest, alpha, false);
+                    src   += 8;
+                    dest  += 8;
+                    alpha += 8;
+                    x     += 8;
+                }
+            }
+
+            if ((y & 1) == 1)
+                continue;
+
+            int uvbegx   = begx >> 1;
+            int uvsrcoff = (y >> 1) * (surface->width >> 1);
+            int udestoff = (y >> 1) * ustride;
+            int vdestoff = (y >> 1) * vstride;
+
+            unsigned char *usrc  = surface->u + uvsrcoff + uvbegx;
+            unsigned char *udest = uptrdest   + udestoff + uvbegx;
+            unsigned char *vsrc  = surface->v + uvsrcoff + uvbegx;
+            unsigned char *vdest = vptrdest   + vdestoff + uvbegx;
+
+            for (int x = begx; x <= endx;)
+            {
+                alpha = surface->alpha + ysrcoff + x;
+                if (x + 16 >= endx)
+                {
+                    if (*alpha != 0)
+                    {
+                        *udest = blendColorsAlpha(*usrc, *udest, *alpha);
+                        *vdest = blendColorsAlpha(*vsrc, *vdest, *alpha);
+                    }
+
+                    usrc++;
+                    udest++;
+                    vsrc++;
+                    vdest++;
+                    x += 2;
+                }
+                else
+                {
+                    blender(usrc, udest, alpha, true);
+                    blender(vsrc, vdest, alpha, true);
+                    usrc += 8;
+                    udest += 8;
+                    vsrc += 8;
+                    vdest += 8;
+                    x += 16;
+                }
+            }
+        }
+    }
+}
+
+static void BlendToBlack(unsigned char *argbptr, uint width, uint outheight)
+{
+    unsigned int i, argb, a, r, g, b;
+    for (i = 0; i < width * outheight; i++)
+    {
+        argb = ((uint*)argbptr)[i];
+        a = (argb >> 24);
+        if (argb && a < 250)
+        {
+            r = (argb >> 16) & 0xff;
+            g = (argb >> 8) & 0xff;
+            b = (argb) & 0xff;
+
+            RGBA_OUT(&argbptr[i * 4], (a * r) >> 8, (a * g) >> 8, 
+                     (a * b) >> 8, 128);
+        }
+    }
+}
+
+/** \fn OSDSurface::BlendToARGB(unsigned char *,uint,uint,bool,uint) const
+ *  \brief Alpha blends OSDSurface to ARGB buffer.
+ *
+ *  \todo Currently blend_to_black is implemented as a post process
+ *        on the whole frame, it would make sense to make this more
+ *        efficient by integrating it into the process.
+ *
+ *  \param stride         Length of each line in output buffer in bytes
+ *  \param outheight      Number of lines in output buffer
+ *  \param blend_to_black Uses Alpha to blend buffer to black
+ *  \param threshold      Alpha threshold above which to perform blending
+ */
+void OSDSurface::BlendToARGB(unsigned char *argbptr, uint stride, 
+                             uint outheight, bool blend_to_black,
+                             uint threshold) const
+{
+    QMutexLocker lock(&usedRegionsLock);
+    const OSDSurface *surface = this;
+    blendtoargb_8_fun blender = blendtoargb_8_init(surface);
+    const unsigned char *cm = surface->cm;
+
+    if (blend_to_black)
+        bzero(argbptr, stride * outheight);
+
+    QMemArray<QRect> rects = surface->usedRegions.rects();
+    QMemArray<QRect>::Iterator it = rects.begin();
+    for (; it != rects.end(); ++it)
+    {
+        QRect drawRect = *it;
+
+        int startcol  = std::max(drawRect.left(),   0);
+        int startline = std::max(drawRect.top(),    0);
+        int endcol    = std::min(drawRect.right(),  width  - 1);
+        int endline   = std::min(drawRect.bottom(), height - 1);
+
+        unsigned char *src, *usrcbase, *vsrcbase, *usrc, *vsrc;
+        unsigned char *dest;
+        unsigned char *alpha;
+
+        int yoffset;
+        int destyoffset;
+
+        int cb, cr, r_add, g_add, b_add;
+        int r, g, b, y0;
+
+        for (int y = startline; y <= endline; y++)
+        {
+            yoffset = y * surface->width;
+            destyoffset = y * stride;
+
+            src = surface->y + yoffset + startcol;
+            dest = argbptr + destyoffset + startcol * 4;
+            alpha = surface->alpha + yoffset + startcol;
+
+            usrcbase = surface->u + (y / 2) * (surface->width / 2);
+            vsrcbase = surface->v + (y / 2) * (surface->width / 2);
+
+            for (int x = startcol; x <= endcol; x++)
+            {
+                usrc = usrcbase + x / 2;
+                vsrc = vsrcbase + x / 2;
+
+                if (((x + 8) >= endcol) || threshold)
+                {
+                    if (*alpha > threshold)
+                    {
+                        YUV_TO_RGB1(*usrc, *vsrc);
+                        YUV_TO_RGB2(r, g, b, *src);
+                        RGBA_OUT(dest, r, g, b, *alpha);
+                    }
+                    src++;
+                    alpha++;
+                    dest += 4;
+                }
+                else
+                {
+                    blender(surface, src, usrc, vsrc, alpha, dest);
+                    src += 8;
+                    dest += 32;
+                    alpha += 8;
+                    x += 7;
+                } 
+            }
+        }
+    }
+    if (blend_to_black)
+        BlendToBlack(argbptr, stride>>2, outheight);
+}
+
+/** \fn OSDSurface::DitherToI44(unsigned char*,bool,uint,uint) const
+ *  \brief Copies and converts OSDSurface to either a greyscale
+ *         IA44 or AI44 buffer.
+ *  \sa DitherToIA44(unsigned char*,uint,uint) const,
+ *      DitherToAI44(unsigned char*,uint,uint) const.
+ *
+ *  \param outbuf Output buffer
+ *  \param ifirst If true output buffer is AI44, otherwise it is IA44
+ *  \param stride Length of each line in output buffer in bytes
+ *  \param outheight Number of lines in output buffer
+ */
+void OSDSurface::DitherToI44(unsigned char *outbuf, bool ifirst,
+                             uint stride, uint outheight) const
+{
+    QMutexLocker lock(&usedRegionsLock);
+    const OSDSurface *surface = this;
+    int ashift = ifirst ? 0 : 4;
+    int amask = ifirst ? 0x0f : 0xf0;
+
+    int ishift = ifirst ? 4 : 0;
+    int imask = ifirst ? 0xf0 : 0x0f; 
+
+    dithertoia44_8_fun ditherer = dithertoia44_8_init(surface);
+    dither8_context *dcontext = init_dithertoia44_8_context(ifirst);
+
+    bzero(outbuf, stride * outheight);
+
+    QMemArray<QRect> rects = surface->usedRegions.rects();
+    QMemArray<QRect>::Iterator it = rects.begin();
+    for (; it != rects.end(); ++it)
+    {
+        QRect drawRect = *it;
+
+        int startcol, startline, endcol, endline;
+        startcol = drawRect.left();
+        startline = drawRect.top();
+        endcol = drawRect.right();
+        endline = drawRect.bottom();
+
+        if (startline < 0) startline = 0;
+        if (endline >= height) endline = height - 1;
+        if (startcol < 0) endcol = 0;
+        if (endcol >= width) endcol = width - 1;
+
+        unsigned char *src;
+        unsigned char *dest;
+        unsigned char *alpha;
+
+        const unsigned char *dmp;
+
+        int yoffset;
+        int destyoffset;
+
+        int grey;
+
+        for (int y = startline; y <= endline; y++)
+        {
+            yoffset = y * surface->width;
+            destyoffset = y * stride;
+
+            src = surface->y + yoffset + startcol;
+            dest = outbuf + destyoffset + startcol;
+            alpha = surface->alpha + yoffset + startcol;
+
+            dmp = DM[(y) & (DM_HEIGHT - 1)];
+
+            for (int x = startcol; x <= endcol; x++)
+            {
+                if (x + 8 >= endcol)
+                {
+                    if (*alpha != 0)
+                    {
+                        grey = *src + ((dmp[(x & (DM_WIDTH - 1))] << 2) >> 4);
+                        grey = (grey - (grey >> 4)) >> 4;
+
+                        *dest = (((*alpha >> 4) << ashift) & amask) |
+                                (((grey) << ishift) & imask);
+                    }
+                    else
+                        *dest = 0;
+
+                    src++;
+                    dest++;
+                    alpha++;
+                }
+                else
+                {
+                    ditherer(src, dest, alpha, dmp, x, dcontext);
+                    src += 8;
+                    dest += 8;
+                    alpha += 8;
+                    x += 7;
+                }
+            }
+        }
+    }
+
+    delete_dithertoia44_8_context(dcontext);
+}
+
+/** \fn OSDSurface::DitherToIA44(unsigned char*,uint,uint) const
+ *  \brief Copies and converts OSDSurface to a greyscale IA44 buffer.
+ *
+ *  \param outbuf Output buffer
+ *  \param stride Length of each line in output buffer in bytes
+ *  \param outheight Number of lines in output buffer
+ *  \sa DitherToI44(unsigned char*,bool,uint,uint) const,
+ *      DitherToAI44(unsigned char*,uint,uint) const.
+ */
+void OSDSurface::DitherToIA44(unsigned char* outbuf,
+                              uint stride, uint outheight) const
+{
+    DitherToI44(outbuf, false, stride, outheight);
+}
+
+/** \fn OSDSurface::DitherToAI44(unsigned char*,uint,uint) const
+ *  \brief Copies and converts OSDSurface to a greyscale AI44 buffer.
+ *
+ *  \param outbuf Output buffer
+ *  \param stride Length of each line in output buffer in bytes
+ *  \param outheight Number of lines in output buffer
+ *  \sa DitherToI44(unsigned char*,bool,uint,uint) const,
+ *      DitherToIA44(unsigned char*,uint,uint) const.
+ */
+void OSDSurface::DitherToAI44(unsigned char* outbuf,
+                              uint stride, uint outheight) const
+{
+    DitherToI44(outbuf, true, stride, outheight);
+}
