@@ -3,21 +3,25 @@
 #include "recordingprofile.h"
 #include "mythcontext.h"
 #include "proglist.h"
+#include "previouslist.h"
 #include "sr_items.h"
 #include "sr_root.h"
 #include "sr_dialog.h"
 #include "jobqueue.h"
 #include "mythdbcon.h"
+#include "viewschdiff.h"
 
 #include <qlayout.h>
 #include <qlabel.h>
 #include <qapplication.h>
 #include <qregexp.h>
+#include <qsqlquery.h>
 
 // NOTE: if this changes, you _MUST_ update the RecTypePriority function 
 // in recordingtypes.cpp.
 
-ScheduledRecording::ScheduledRecording() 
+ScheduledRecording::ScheduledRecording() :
+    ConfigurationGroup(true, true, false, false)
 {
     m_pginfo = NULL;
     type = NULL;
@@ -27,6 +31,7 @@ ScheduledRecording::ScheduledRecording()
     dupmethod = NULL;
     autoexpire = NULL;
     autotranscode = NULL;
+    transcoder = NULL;
     autocommflag = NULL;
     autouserjob1 = NULL;
     autouserjob2 = NULL;
@@ -39,6 +44,8 @@ ScheduledRecording::ScheduledRecording()
     m_dialog = NULL;
     recpriority = NULL;
     recgroup = NULL;
+    playgroup = NULL;
+    prefinput = NULL;
     inactive = NULL;
     searchType = "";
     searchForWhat = "";
@@ -51,32 +58,39 @@ ScheduledRecording::ScheduledRecording()
     
     addChild(id = new ID());
     
-    channel = new SRChannel(*this);
-    station = new SRStation(*this);
-    title = new SRTitle(*this);
-    subtitle = new SRSubtitle(*this);
-    description = new SRDescription(*this);
-    startTime = new SRStartTime(*this);
-    endTime = new SREndTime(*this);
-    startDate = new SRStartDate(*this);
-    endDate = new SREndDate(*this);
-    category = new SRCategory(*this);
-    seriesid = new SRSeriesid(*this);
-    programid = new SRProgramid(*this);
-    findday = new SRFindDay(*this);
-    findtime = new SRFindTime(*this);
-    findid = new SRFindId(*this);
-    search = new SRRecSearchType(*this);
-    
-    rootGroup = new RootSRGroup(*this);
+    channel = new SRChannel(this);
+    station = new SRStation(this);
+    title = new SRTitle(this);
+    subtitle = new SRSubtitle(this);
+    description = new SRDescription(this);
+    startTime = new SRStartTime(this);
+    endTime = new SREndTime(this);
+    startDate = new SRStartDate(this);
+    endDate = new SREndDate(this);
+    category = new SRCategory(this);
+    seriesid = new SRSeriesid(this);
+    programid = new SRProgramid(this);
+    findday = new SRFindDay(this);
+    findtime = new SRFindTime(this);
+    findid = new SRFindId(this);
+    parentid = new SRParentId(this);
+    search = new SRRecSearchType(this);
+    rootGroup = new RootSRGroup(this);
 }
 
+ScheduledRecording::~ScheduledRecording() 
+{
+    // rootGroup is unique among this class' member variables in
+    // that it's not self-managed
+    if (!rootGroup.isNull())
+        delete rootGroup;
+}
 
-void ScheduledRecording::load(QSqlDatabase *db)
+void ScheduledRecording::load()
 {
     if (getRecordID())
     {
-        ConfigurationGroup::load(db);
+        ConfigurationGroup::load();
         
         QString tmpType = type->getValue();
         type->clearSelections();
@@ -84,43 +98,57 @@ void ScheduledRecording::load(QSqlDatabase *db)
             tmpType.toInt() == kDontRecord)
             type->addOverrideSelections();
         else
-            type->addNormalSelections(!station->getValue().isEmpty());
-        
+            type->addNormalSelections(!station->getValue().isEmpty(),
+                                      search->intValue() == kManualSearch);
+       
         type->setValue(tmpType);
         type->setUnchanged();
-        fetchChannelInfo(db);
+
+        fetchChannelInfo();
     }
 }
 
 
 
-void ScheduledRecording::loadByProgram(QSqlDatabase* db, ProgramInfo* proginfo) 
+void ScheduledRecording::loadByProgram(const ProgramInfo* proginfo) 
 {
     m_pginfo = proginfo;
     
     if (proginfo->recordid)
-        loadByID(db, proginfo->recordid);
+        loadByID(proginfo->recordid);
     else
-        setDefault(db, true);
-        
+        setDefault(true);
 
-    
-    if (search->intValue() == kNoSearch)
-        setProgram(proginfo, db);
+    if (search->intValue() == kNoSearch ||
+        search->intValue() == kManualSearch)
+    {
+        setProgram(proginfo);
+        if (!proginfo->recordid)
+            playgroup->setValue(PlayGroup::GetInitialName(proginfo));
+    }
 }
 
-void ScheduledRecording::loadBySearch(QSqlDatabase *db,
-                                      RecSearchType lsearch,
+void ScheduledRecording::loadBySearch(RecSearchType lsearch,
                                       QString textname,
                                       QString forwhat)
 {
-    int rid = 0;
-    QString thequery = QString("SELECT recordid FROM record WHERE "
-                               "search = %1 AND description LIKE '%2'")
-        .arg(lsearch).arg(forwhat);
-    QSqlQuery query = db->exec(thequery);
+    QString from = "";
+    loadBySearch(lsearch, textname, from, forwhat);
+}
+void ScheduledRecording::loadBySearch(RecSearchType lsearch,
+                                      QString textname,
+                                      QString from,
+                                      QString forwhat)
+{
+    MSqlQuery query(MSqlQuery::InitCon());
 
-    if (query.isActive())
+    int rid = 0;
+    query.prepare("SELECT recordid FROM record WHERE "
+                  "search = :SEARCH AND description LIKE :FORWHAT");
+    query.bindValue(":SEARCH", lsearch);
+    query.bindValue(":FORWHAT", forwhat);
+
+    if (query.exec() && query.isActive())
     {
         if (query.next())
             rid = query.value(0).toInt();
@@ -129,10 +157,10 @@ void ScheduledRecording::loadBySearch(QSqlDatabase *db,
         MythContext::DBError("loadBySearch", query);
 
     if (rid)
-        loadByID(db, rid);
+        loadByID(rid);
     else
     {
-        setDefault(db, false);
+        setDefault(false);
         search->setValue(lsearch);
         searchForWhat = forwhat;
         
@@ -156,26 +184,58 @@ void ScheduledRecording::loadBySearch(QSqlDatabase *db,
         }
         QString ltitle = QString("%1 %2").arg(textname).arg(searchType);
         title->setValue(ltitle);
+        subtitle->setValue(from);
         description->setValue(forwhat);
+        findday->setValue((startDate->dateValue().dayOfWeek() + 1) % 7);
+        QDate epoch = QDate::QDate (1970, 1, 1);
+        findid->setValue(epoch.daysTo(startDate->dateValue()) + 719528);
     } 
 }
 
-void ScheduledRecording::fetchChannelInfo(QSqlDatabase *db)
+void ScheduledRecording::modifyPowerSearchByID(int rid,
+                                               QString textname,
+                                               QString forwhat)
+{
+    QString from = "";
+    modifyPowerSearchByID(rid, textname, from, forwhat);
+}
+
+void ScheduledRecording::modifyPowerSearchByID(int rid,
+                                               QString textname,
+                                               QString from,
+                                               QString forwhat)
+{
+    if (rid <= 0)
+        return;
+
+    loadByID(rid);
+    if (search->intValue() != kPowerSearch)
+        return;
+
+    QString ltitle = textname + " (" + QObject::tr("Power Search") + ")";
+    title->setValue(ltitle);
+    subtitle->setValue(from);
+    description->setValue(forwhat);
+}
+
+void ScheduledRecording::fetchChannelInfo()
 {
     
     if (channel->getValue().toInt() > 0)
     {
+        MSqlQuery query(MSqlQuery::InitCon());
+
         QString queryStr(QString("SELECT channum, callsign, name FROM channel "
                                  "WHERE chanid = '%1';").arg(channel->getValue()));
     
-        QSqlQuery result = db->exec(queryStr);
+        query.prepare(queryStr);
         
-        if (result.isActive() && result.numRowsAffected() > 0)
+        if (query.exec() && query.isActive() && query.size() > 0)
         {
-            result.next();
-            chanstr =  result.value(0).toString();
-            chansign =  result.value(1).toString();
-            channame =  result.value(2).toString();
+            query.next();
+            chanstr =  query.value(0).toString();
+            chansign =  query.value(1).toString();
+            channame =  query.value(2).toString();
         }
     }
     else
@@ -202,10 +262,11 @@ void ScheduledRecording::ToMap(QMap<QString, QString>& progMap)
 {
     QString searchtitle = "";
 
-    if (m_pginfo && search->intValue() != kNoSearch)
+    if (m_pginfo && search->intValue() != kNoSearch &&
+                    search->intValue() != kManualSearch)
     {
         searchtitle = title->getValue();
-        m_pginfo->ToMap(NULL, progMap);
+        m_pginfo->ToMap(progMap);
     }
     else
     {
@@ -268,8 +329,9 @@ void ScheduledRecording::ToMap(QMap<QString, QString>& progMap)
         QString findfrom = findtime->timeValue().toString(timeFormat);
         if (recType == kFindWeeklyRecord)
         {
+            int daynum = (findday->intValue() + 5) % 7 + 1;
             findfrom = QString("%1, %2")
-                               .arg(QDate::shortDayName(findday->intValue()-1))
+                               .arg(QDate::shortDayName(daynum))
                                .arg(findfrom);
         }
         progMap["subtitle"] = QString("(%1 %2) %3")
@@ -299,9 +361,9 @@ void ScheduledRecording::ToMap(QMap<QString, QString>& progMap)
 }
 
 
-void ScheduledRecording::loadByID(QSqlDatabase* db, int recordID) {
+void ScheduledRecording::loadByID(int recordID) {
     id->setValue(recordID);
-    load(db);
+    load();
 }
 
 RecordingType ScheduledRecording::getRecordingType(void) const {
@@ -316,14 +378,32 @@ RecSearchType ScheduledRecording::getSearchType(void) const {
     return (RecSearchType)(search->intValue());
 }
 
-bool ScheduledRecording::GetAutoExpire(void) const {
+void ScheduledRecording::setSearchType(RecSearchType newType) {
+    if (type->getValue().toInt() == kOverrideRecord ||
+        type->getValue().toInt() == kDontRecord)
+    {
+        VERBOSE(VB_IMPORTANT, "Attempt to set search type for "
+                "override recording");
+        return;
+    }
+    search->setValue(newType);
+    type->clearSelections();
+    type->addNormalSelections(!station->getValue().isEmpty(),
+                              search->intValue() == kManualSearch);
+}
+
+int ScheduledRecording::GetAutoExpire(void) const {
     return(autoexpire->getValue().toInt());
 }
 
-void ScheduledRecording::SetAutoExpire(bool expire) {
+void ScheduledRecording::SetAutoExpire(int expire) {
     autoexpire->setValue(expire);
 }
 
+int ScheduledRecording::GetTranscoder(void) const {
+    return transcoder->getValue().toInt();
+}
+
 int ScheduledRecording::GetAutoRunJobs(void) const {
     int result = 0;
 
@@ -351,32 +431,74 @@ bool ScheduledRecording::GetMaxNewest(void) const {
     return(maxnewest->getValue().toInt());
 }
 
-void ScheduledRecording::save(QSqlDatabase* db) 
+void ScheduledRecording::save(void)
+{
+    // NOTE: we can not use a default value for send(bool) because
+    // anyone calling save on a parent type pointer will then not 
+    // use our version of the virtual funcation.
+    save(true);
+}
+
+void ScheduledRecording::save(bool sendSig)
 {
     if (type->isChanged() && getRecordingType() == kNotRecording)
     {
-        remove(db);
+        remove();
     }
     else
     {
-        ConfigurationGroup::save(db);
+        ConfigurationGroup::save();
+
+        MSqlQuery query(MSqlQuery::InitCon());
+        query.prepare(
+            "UPDATE recorded "
+                "SET recpriority = :RECPRIORITY, "
+                    "transcoder = :TRANSCODER, playgroup = :PLAYGROUP "
+                "WHERE recordid = :RECORDID ;");
+        query.bindValue(":RECPRIORITY", getRecPriority());
+        query.bindValue(":TRANSCODER", transcoder->getValue().toInt());
+        query.bindValue(":PLAYGROUP", playgroup->getValue());
+        query.bindValue(":RECORDID", getRecordID());
+
+        if (!query.exec())
+            MythContext::DBError("UPDATE recorded", query);
     }
-    signalChange(getRecordID());
+    if (sendSig)
+        signalChange(getRecordID());
 }
 
-void ScheduledRecording::remove(QSqlDatabase* db) {
-    if (!getRecordID())
+void ScheduledRecording::save(QString destination) 
+{
+    if (type->isChanged() && getRecordingType() == kNotRecording)
+    {
+        remove();
+    }
+    else
+    {
+        ConfigurationGroup::save(destination);
+    }
+}
+
+void ScheduledRecording::remove() 
+{
+    int rid = getRecordID();
+
+    if (!rid)
         return;
 
-    QString query = QString("DELETE FROM record WHERE recordid = %1")
-                            .arg(getRecordID());
-    db->exec(query);
-    query = QString("UPDATE recordid FROM recorded SET recordid = NULL "
-                    "WHERE recordid = %1")
-                    .arg(getRecordID());
+    QString querystr;
+    MSqlQuery query(MSqlQuery::InitCon());
+    querystr = QString("DELETE FROM record WHERE recordid = %1").arg(rid);
+    query.prepare(querystr);
+    query.exec();
+
+    querystr = QString("DELETE FROM oldfind WHERE recordid = %1").arg(rid);
+    query.prepare(querystr);
+    query.exec();
 }
 
-void ScheduledRecording::signalChange(int recordid) {
+void ScheduledRecording::signalChange(int recordid) 
+{
     if (gContext->IsBackend())
     {
         MythEvent me(QString("RESCHEDULE_RECORDINGS %1").arg(recordid));
@@ -387,155 +509,103 @@ void ScheduledRecording::signalChange(int recordid) {
         QStringList slist;
         slist << QString("RESCHEDULE_RECORDINGS %1").arg(recordid);
         if (!gContext->SendReceiveStringList(slist))
-            cerr << "error resceduling id " << recordid << 
-                " in ScheduledRecording::signalChange" << endl;
+            VERBOSE(VB_IMPORTANT, QString("Error rescheduling id %1 "
+                                    "in ScheduledRecording::signalChange")
+                                    .arg(recordid));
     }
 }
 
-void ScheduledRecording::doneRecording(QSqlDatabase* db, 
-                                       const ProgramInfo& proginfo) 
+void ScheduledRecording::doneRecording(ProgramInfo& proginfo) 
 {
-    if (getRecordingType() == kFindOneRecord)
-        remove(db);
+    proginfo.recstatus = rsRecorded;
 
-    QString msg;
-  
-    msg = QString("Finished recording %1 on channel: %2")
-                  .arg(proginfo.title)
-                  .arg(proginfo.chanid);
+    QString msg = "Finished recording";
+    QString subtitle = proginfo.subtitle.isEmpty() ? "" :
+                                    QString(" \"%1\"").arg(proginfo.subtitle);
+    QString details = QString("%1%2: channel %3")
+                                    .arg(proginfo.title)
+                                    .arg(subtitle)
+                                    .arg(proginfo.chanid);
 
-    VERBOSE(VB_GENERAL, msg.local8Bit());
-    gContext->LogEntry("scheduler", LP_NOTICE, "Finished recording", 
-                       msg.utf8());
-
-    addHistory(db, proginfo);
+    VERBOSE(VB_GENERAL, QString("%1 %2").arg(msg).arg(details).local8Bit());
+    gContext->LogEntry("scheduler", LP_NOTICE, msg, details);
 }
 
-void ScheduledRecording::addHistory(QSqlDatabase* db, 
-                                    const ProgramInfo& proginfo) 
-{
-    MSqlQuery result(QString::null, db);
-
-    result.prepare("REPLACE INTO oldrecorded (chanid,starttime,"
-                   "endtime,title,subtitle,description,category,"
-                   "seriesid,programid,findid) "
-                   "VALUES(:CHANID,:START,:END,:TITLE,:SUBTITLE,:DESC,"
-                   ":CATEGORY,:SERIESID,:PROGRAMID,:FINDID);");
-    result.bindValue(":CHANID", proginfo.chanid);
-    result.bindValue(":START", proginfo.recstartts.toString(Qt::ISODate));
-    result.bindValue(":END", proginfo.recendts.toString(Qt::ISODate));
-    result.bindValue(":TITLE", proginfo.title.utf8());
-    result.bindValue(":SUBTITLE", proginfo.subtitle.utf8());
-    result.bindValue(":DESC", proginfo.description.utf8());
-    result.bindValue(":CATEGORY", proginfo.category.utf8());
-    result.bindValue(":SERIESID", proginfo.seriesid.utf8());
-    result.bindValue(":PROGRAMID", proginfo.programid.utf8());
-    result.bindValue(":FINDID", proginfo.findid);
-
-    result.exec();
-
-    if (!result.isActive())
-    {
-        MythContext::DBError("addHistory", result);
-    }
-    else
-    {
-        // The addition of an entry to oldrecorded may affect near-future
-        // scheduling decisions, so recalculate
-        signalChange(0);
-    }
-}
-
-void ScheduledRecording::forgetHistory(QSqlDatabase* db,
-                                       const ProgramInfo& proginfo)
-{
-    MSqlQuery result(QString::null, db);
-
-    result.prepare("DELETE FROM oldrecorded WHERE title = :TITLE AND "
-                   "((subtitle = :SUBTITLE AND description = :DESC) OR "
-                   "(programid <> '' AND programid = :PROGRAMID) OR "
-                   "(findid <> 0 AND findid = :FINDID))");
-    result.bindValue(":TITLE", proginfo.title.utf8());
-    result.bindValue(":SUBTITLE", proginfo.subtitle.utf8());
-    result.bindValue(":DESC", proginfo.description.utf8());
-    result.bindValue(":PROGRAMID", proginfo.programid);
-    result.bindValue(":FINDID", proginfo.findid);
-    
-    result.exec();
-    if (!result.isActive())
-    {
-        MythContext::DBError("forgetHistory", result);
-    }
-    else
-    {
-        // The removal of an entry from oldrecorded may affect near-future
-        // scheduling decisions, so recalculate
-        signalChange(0);
-    }
-}
-
-
-
-void ScheduledRecording::runProgList(void)
+void ScheduledRecording::runTitleList(void)
 {
     ProgLister *pl = NULL;
 
-    if(search->intValue() && getRecordID())
+    if (search->intValue())
     {
-        ScheduledRecording rule;
-        rule.loadByID(QSqlDatabase::database(), getRecordID());
-
-        switch (search->intValue())
+        if (m_pginfo)
         {
-        case kTitleSearch:
-            pl = new ProgLister(plTitleSearch, rule.description->getValue(),
-                                QSqlDatabase::database(),
+            pl = new ProgLister(plTitle, m_pginfo->title, "",
                                 gContext->GetMainWindow(), "proglist");
-            break;
-        case kKeywordSearch:
-            pl = new ProgLister(plKeywordSearch, rule.description->getValue(),
-                                QSqlDatabase::database(),
+        }
+        else
+        {
+            QString trimTitle = title->getValue();
+            trimTitle.remove(QRegExp(" \\(.*\\)$"));
+            pl = new ProgLister(plTitle, trimTitle, "",
                                 gContext->GetMainWindow(), "proglist");
-            break;
-        case kPeopleSearch:
-            pl = new ProgLister(plPeopleSearch, rule.description->getValue(),
-                                QSqlDatabase::database(),
-                                gContext->GetMainWindow(), "proglist");
-            break;
-        case kPowerSearch:
-            pl = new ProgLister(plSQLSearch, rule.description->getValue(),
-                                QSqlDatabase::database(),
-                                gContext->GetMainWindow(), "proglist");
-            break;
-        default:
-            pl = new ProgLister(plTitle, title->getValue(),
-                                QSqlDatabase::database(),
-                                gContext->GetMainWindow(), "proglist");
-            break;
         }
     }
     else
-        pl = new ProgLister(plTitle, title->getValue(),
-                            QSqlDatabase::database(),
+    {
+        pl = new ProgLister(plTitle, title->getValue(), "",
                             gContext->GetMainWindow(), "proglist");
+    }
     pl->exec();
     delete pl;
 }
 
+void ScheduledRecording::runRuleList(void)
+{
+    ProgLister *pl = NULL;
+
+    if (getRecordID())
+    {
+        pl = new ProgLister(plRecordid,
+                            QString("%1").arg(getRecordID()), "",
+                            gContext->GetMainWindow(), "proglist");
+    }
+    else
+    {
+        pl = new ProgLister(plTitle, title->getValue(), "",
+                            gContext->GetMainWindow(), "proglist");
+    }
+    pl->exec();
+    delete pl;
+}
+
+void ScheduledRecording::runPrevList(void)
+{
+    PreviousList *pb = NULL;
+
+    pb = new PreviousList(gContext->GetMainWindow(), "proglist",
+                          getRecordID(), title->getValue());
+    pb->exec();
+    delete pb;
+}
+
 void ScheduledRecording::runShowDetails(void)
 {
     if (m_pginfo)
-        m_pginfo->showDetails(QSqlDatabase::database());
+        m_pginfo->showDetails();
 }
 
-void ScheduledRecording::fillSelections(QSqlDatabase* db, SelectSetting* setting) {
-    QSqlQuery result = db->exec("SELECT recordid FROM record");
-    if (result.isActive() && result.numRowsAffected() > 0)
-        while (result.next()) {
+void ScheduledRecording::fillSelections(SelectSetting* setting) 
+{
+    MSqlQuery result(MSqlQuery::InitCon());
+    result.prepare("SELECT recordid FROM record");
+    if (result.exec() && result.isActive() && result.size() > 0)
+    {
+        while (result.next()) 
+        {
             int recid = result.value(0).toInt();
 
             ScheduledRecording sr;
-            sr.loadByID(db, recid);
+            sr.loadByID(recid);
 
             QString label;
             QString weekly = "";
@@ -586,16 +656,19 @@ void ScheduledRecording::fillSelections(QSqlDatabase* db, SelectSetting* setting
 
             setting->addSelection(label, QString::number(recid));
         }
+    }
 }
 
-void ScheduledRecordingEditor::load(QSqlDatabase* db) {
+void ScheduledRecordingEditor::load() 
+{
     clearSelections();
-    ScheduledRecording::fillSelections(db, this);
+    ScheduledRecording::fillSelections(this);
 }
 
 
-int ScheduledRecordingEditor::exec(QSqlDatabase* db) {
-    while (ConfigurationDialog::exec(db) == QDialog::Accepted)
+int ScheduledRecordingEditor::exec() 
+{
+    while (ConfigurationDialog::exec() == QDialog::Accepted)
         open(getValue().toInt());
 
     return QDialog::Rejected;
@@ -605,10 +678,10 @@ void ScheduledRecordingEditor::open(int id) {
     ScheduledRecording* sr = new ScheduledRecording();
 
     if (id != 0)
-        sr->loadByID(db,id);
+        sr->loadByID(id);
 
-    if (sr->exec(db) == QDialog::Accepted)
-        sr->save(db);
+    if (sr->exec() == QDialog::Accepted)
+        sr->save();
     delete sr;
 }
 
@@ -622,6 +695,10 @@ void ScheduledRecording::setEnd(const QDateTime& end) {
     endDate->setValue(end.date());
 }
 
+int ScheduledRecording::getRecPriority(void) const {
+    return recpriority->getValue().toInt();
+}
+
 void ScheduledRecording::setRecPriority(int newrecpriority) {
     recpriority->setValue(newrecpriority);
 }
@@ -630,23 +707,49 @@ void ScheduledRecording::setRecGroup(const QString& newrecgroup) {
     recgroup->setValue(newrecgroup);
 }
 
+QString ScheduledRecording::GetRecGroup(void) const {
+    return recgroup->getValue();
+}
+
+void ScheduledRecording::setPlayGroup(const QString& newplaygroup) {
+    playgroup->setValue(newplaygroup);
+}
+
 QString ScheduledRecording::getProfileName(void) const {
     return profile->getValue();
 }
 
+QString ScheduledRecording::getRecordTitle(void) const {
+    return title->getValue();
+}
 
+QString ScheduledRecording::getRecordSubTitle(void) const {
+    return subtitle->getValue();
+}
 
-MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent, const char *name)
+QString ScheduledRecording::getRecordDescription(void) const {
+    return description->getValue();
+}
+
+MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent,
+                                             const char *name)
 {
-
+    (void) parent;
+    (void) name;
+#ifdef USING_FRONTEND
     MythDialog* dlg = new RecOptDialog(this, parent, name);
     rootGroup->setDialog(dlg);
     return dlg;
+#else
+    VERBOSE(VB_IMPORTANT, "You must compile the frontend "
+            "to use a dialogWidget.");
+    return NULL;
+#endif
 }
 
 
 
-void ScheduledRecording::setDefault(QSqlDatabase *db, bool haschannel)
+void ScheduledRecording::setDefault(bool haschannel)
 {
     id->setValue(0);
     title->setValue("");
@@ -665,6 +768,7 @@ void ScheduledRecording::setDefault(QSqlDatabase *db, bool haschannel)
     findday->setValue(-1);
     findtime->setValue(QTime::fromString("00:00:00", Qt::ISODate));
     findid->setValue(0);
+    parentid->setValue(0);
     category->setValue("");
     search->setValue(kNoSearch);
 
@@ -677,37 +781,43 @@ void ScheduledRecording::setDefault(QSqlDatabase *db, bool haschannel)
     }
     
     type->clearSelections();
-    type->addNormalSelections(haschannel);
+    type->addNormalSelections(haschannel, search->intValue() == kManualSearch);
     type->setValue(kNotRecording);
     
-    profile->fillSelections(db);
+    profile->fillSelections();
     profile->setValue(QObject::tr("Default"));
     
     dupin->setValue(kDupsInAll);
     dupmethod->setValue(kDupCheckSubDesc);
     maxepisodes->setValue(0);
-    startoffset->setValue(0);
-    endoffset->setValue(0);   
+    startoffset->setValue(gContext->GetNumSetting("DefaultStartOffset", 0));
+    endoffset->setValue(gContext->GetNumSetting("DefaultEndOffset", 0));
     maxnewest->setValue(0);
     recpriority->setValue(0);
     
     autoexpire->setValue(gContext->GetNumSetting("AutoExpireDefault", 0));
 
-    autotranscode->setValue(0);
+    autotranscode->setValue(gContext->GetNumSetting("AutoTranscode", 0));
+    transcoder->fillSelections();
+    transcoder->setValue(gContext->GetNumSetting("DefaultTranscoder",
+                         RecordingProfile::TranscoderAutodetect));
     autocommflag->setValue(gContext->GetNumSetting("AutoCommercialFlag", 1));
+    autouserjob1->setValue(gContext->GetNumSetting("AutoRunUserJob1", 0));
+    autouserjob2->setValue(gContext->GetNumSetting("AutoRunUserJob2", 0));
+    autouserjob3->setValue(gContext->GetNumSetting("AutoRunUserJob3", 0));
+    autouserjob4->setValue(gContext->GetNumSetting("AutoRunUserJob4", 0));
 
-    autouserjob1->setValue(0);
-    autouserjob2->setValue(0);
-    autouserjob3->setValue(0);
-    autouserjob4->setValue(0);
-
-    recgroup->fillSelections(db);    
+    recgroup->fillSelections();    
     recgroup->setValue("Default");
+    playgroup->fillSelections();    
+    playgroup->setValue("Default");
+    prefinput->fillSelections();    
+    prefinput->setValue(0);
 
     inactive->setValue(0);
 }
 
-void ScheduledRecording::setProgram(ProgramInfo *proginfo, QSqlDatabase* db)
+void ScheduledRecording::setProgram(const ProgramInfo *proginfo)
 {
     m_pginfo = proginfo;
     if (proginfo)
@@ -727,12 +837,24 @@ void ScheduledRecording::setProgram(ProgramInfo *proginfo, QSqlDatabase* db)
         {
             findday->setValue((proginfo->startts.date().dayOfWeek() + 1) % 7);
             findtime->setValue(proginfo->startts.time());
+
+            QDate epoch = QDate::QDate (1970, 1, 1);
+            findid->setValue(epoch.daysTo(proginfo->startts.date()) + 719528);
+        }
+        else
+        {
+            if (findid->intValue() > 0)
+                findid->setValue(proginfo->findid);
+            else
+            {
+                QDate epoch = QDate::QDate (1970, 1, 1);
+                findid->setValue(epoch.daysTo(proginfo->startts.date()) +
+                                              719528);
+            }
         }
-        findid->setValue(proginfo->findid);
         category->setValue(proginfo->category);
         
-        if (db)
-            fetchChannelInfo(db);
+        fetchChannelInfo();
     }
 }
 
@@ -745,10 +867,18 @@ void ScheduledRecording::makeOverride(void)
     id->setValue(0);
     type->clearSelections();
     type->addOverrideSelections();
-    search->setValue(kNoSearch);
+    type->setValue(kNotRecording);
+    inactive->setValue(0);
+
+    if (search->intValue() == kManualSearch)
+        search->setChanged();
+    else
+        search->setValue(kNoSearch);
 
     setProgram(m_pginfo);
 
+    parentid->setValue(m_pginfo->recordid);
+
     profile->setChanged();
     dupin->setChanged();
     dupmethod->setChanged();
@@ -765,9 +895,98 @@ void ScheduledRecording::makeOverride(void)
     endoffset->setChanged();
     recpriority->setChanged();
     recgroup->setChanged();
-    inactive->setChanged();
+    playgroup->setChanged();
 }
 
+void
+ScheduledRecording::testRecording()
+{
+
+    QString ttable = "record_tmp";
+
+    MSqlQueryInfo dbcon = MSqlQuery::SchedCon();
+    MSqlQuery query(dbcon);
+    QString thequery;
+
+    thequery ="SELECT GET_LOCK(:LOCK, 2);";
+    query.prepare(thequery);
+    query.bindValue(":LOCK", "DiffSchedule");
+    query.exec();
+    if (query.lastError().type() != QSqlError::None)
+    {
+        QString msg =
+            QString("DB Error (Obtaining lock in testRecording): \n"
+                    "Query was: %1 \nError was: %2 \n")
+            .arg(thequery)
+            .arg(MythContext::DBErrorMessage(query.lastError()));
+        VERBOSE(VB_IMPORTANT, msg);
+        return;
+    }
 
 
+    thequery = QString("DROP TABLE IF EXISTS %1;").arg(ttable);
+    query.prepare(thequery);
+    query.exec();
+    if (query.lastError().type() != QSqlError::None)
+    {
+        QString msg =
+            QString("DB Error (deleting old table in testRecording): \n"
+                    "Query was: %1 \nError was: %2 \n")
+            .arg(thequery)
+            .arg(MythContext::DBErrorMessage(query.lastError()));
+        VERBOSE(VB_IMPORTANT, msg);
+        return;
+    }
 
+    thequery = QString("CREATE TABLE %1 SELECT * FROM record;").arg(ttable);
+    query.prepare(thequery);
+    query.exec();
+    if (query.lastError().type() != QSqlError::None)
+    {
+        QString msg =
+            QString("DB Error (create new table): \n"
+                    "Query was: %1 \nError was: %2 \n")
+            .arg(thequery)
+            .arg(MythContext::DBErrorMessage(query.lastError()));
+        VERBOSE(VB_IMPORTANT, msg);
+        return;
+    }
+
+    bool resetID = false;
+    if (getRecordID() == 0) {
+        thequery = QString("SELECT MAX(recordid) FROM %1 ORDER BY recordid;")
+                           .arg(ttable);
+        query.prepare(thequery);
+        query.exec();
+        if (query.isActive() && query.next())
+            id->setValue(query.value(0).toInt() + 1);
+        else
+            id->setValue(100000);
+    }
+    save(ttable);
+
+    ViewScheduleDiff vsd(gContext->GetMainWindow(), "Preview Schedule Changes",
+                         ttable, getRecordID(), title->getValue());
+
+    thequery = "SELECT RELEASE_LOCK(:LOCK);";
+    query.prepare(thequery);
+    query.bindValue(":LOCK", "DiffSchedule");
+    query.exec();
+    if (query.lastError().type() != QSqlError::None)
+    {
+        QString msg =
+            QString("DB Error (free lock): \n"
+                    "Query was: %1 \nError was: %2 \n")
+            .arg(thequery)
+            .arg(MythContext::DBErrorMessage(query.lastError()));
+        VERBOSE(VB_IMPORTANT, msg);
+        return;
+    }
+
+    if (resetID)
+        id->setValue(0);
+
+    vsd.exec();
+}
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
