@@ -16,21 +16,23 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <sys/ioctl.h>
-#include <errno.h>
+#include <cstdio>
+#include <cerrno>
+#include <cmath>
 #include <unistd.h>
+#include <fcntl.h>
+
+#include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 #include <sys/poll.h>
-#include <stdio.h>
-#include <math.h>
-#include <string.h>
-#include <qstring.h>
-#include <iostream>
-#include <sstream>
 
 #include <mythcontext.h>
+#include "mythdialogs.h" // for OpenGL VSync
+#ifdef USING_XV
+#include "videoout_xv.h" // for OpenGL VSync
+#endif
+#include "util-x11.h"    // for OpenGL VSync
 
 #ifdef USING_OPENGL_VSYNC
 #define GLX_GLXEXT_PROTOTYPES
@@ -52,51 +54,66 @@ bool tryingVideoSync = false;
 int VideoSync::m_forceskip = 0;
 
 #define TESTVIDEOSYNC(NAME) \
-    if (++m_forceskip > skip) \
+    do { if (++m_forceskip > skip) \
     { \
-        trial = new NAME (frame_interval, refresh_interval, interlaced); \
+        trial = new NAME (video_output,     frame_interval, \
+                          refresh_interval, halve_frame_interval); \
         if (trial->TryInit()) \
         { \
 	    m_forceskip = skip; \
-	    tryingVideoSync=false; \
+	    tryingVideoSync = false; \
 	    return trial; \
         } \
         delete trial; \
-    }
+    } } while (false)
 
-VideoSync *VideoSync::BestMethod(int frame_interval, int refresh_interval,
-                                 bool interlaced)
+/** \fn VideoSync::BestMethod(VideoOutput*,uint,uint,bool)
+ *  \brief Returns the most sophisticated video sync method available.
+ */
+VideoSync *VideoSync::BestMethod(VideoOutput *video_output,
+                                 uint frame_interval, uint refresh_interval,
+                                 bool halve_frame_interval)
 {
-    VideoSync *trial;
+    VideoSync *trial = NULL;
+    tryingVideoSync  = true;
+    bool tryOpenGL   = (gContext->GetNumSetting("UseOpenGLVSync", 1) &&
+                        (getenv("NO_OPENGL_VSYNC") == NULL));
 
-    tryingVideoSync=true;
-    int skip=0;
-    if (m_forceskip!=0)
+    // m_forceskip allows for skipping one sync method
+    // due to crash on the previous run.
+    int skip = 0;
+    if (m_forceskip)
     {
         VERBOSE(VB_PLAYBACK, QString("A previous trial crashed,"
 		" skipping %1").arg(m_forceskip));
     
-	skip=m_forceskip;
-	m_forceskip=0;
+	skip = m_forceskip;
+	m_forceskip = 0;
     }
     
-    TESTVIDEOSYNC(nVidiaVideoSync)
-    TESTVIDEOSYNC(DRMVideoSync)
-#ifdef USING_OPENGL_VSYNC
-    TESTVIDEOSYNC(OpenGLVideoSync)
-#endif
+    TESTVIDEOSYNC(nVidiaVideoSync);
+    TESTVIDEOSYNC(DRMVideoSync);
+    if (tryOpenGL)
+        TESTVIDEOSYNC(OpenGLVideoSync);
 #ifdef __linux__
-    TESTVIDEOSYNC(RTCVideoSync)
+    TESTVIDEOSYNC(RTCVideoSync);
 #endif
-    TESTVIDEOSYNC(BusyWaitVideoSync)
+    TESTVIDEOSYNC(BusyWaitVideoSync);
 
     tryingVideoSync=false;
     return NULL;
 }
 
-VideoSync::VideoSync(int frameint, int refreshint, bool interlaced) :
-    m_frame_interval(frameint), m_refresh_interval(refreshint), 
-    m_interlaced(interlaced) 
+/** \fn VideoSync::VideoSync(VideoOutput*,int,int,bool)
+ *  \brief Used by BestMethod(VideoOutput*,uint,uint,bool) to initialize
+ *         video synchronization method.
+ */
+VideoSync::VideoSync(VideoOutput *video_output,
+                     int frameint, int refreshint,
+                     bool halve_frame_interval) :
+    m_video_output(video_output),
+    m_frame_interval(frameint), m_refresh_interval(refreshint),
+    m_interlaced(halve_frame_interval) 
 {
     if (m_interlaced && m_refresh_interval > m_frame_interval/2)
         m_interlaced = false; // can't display both fields at 2x rate
@@ -104,11 +121,14 @@ VideoSync::VideoSync(int frameint, int refreshint, bool interlaced) :
     //cout << "Frame interval: " << m_frame_interval << endl;
 }
 
-void VideoSync::Start()
+void VideoSync::Start(void)
 {
     gettimeofday(&m_nexttrigger, NULL); // now
 }
 
+/** \fn VideoSync::SetFrameInterval(int fr, bool intr)
+ *  \brief Change frame interval and interlacing attributes
+ */
 void VideoSync::SetFrameInterval(int fr, bool intr)
 {
     m_frame_interval = fr;
@@ -135,6 +155,11 @@ void VideoSync::OffsetTimeval(struct timeval& tv, int offset)
     }
 }
 
+/** \fn VideoSync::UpdateNexttrigger()
+ *  \brief Internal method to tells video synchronization method to use
+ *         the next frame (or field, if interlaced) for CalcDelay() 
+ *         and WaitForFrame().
+ */
 void VideoSync::UpdateNexttrigger()
 {
     // Offset by frame interval -- if interlaced, only delay by half
@@ -145,6 +170,15 @@ void VideoSync::UpdateNexttrigger()
         OffsetTimeval(m_nexttrigger, m_frame_interval);
 }
 
+/** \fn VideoSync::CalcDelay()
+ *  \brief Calculates the delay to the next frame.
+ *
+ *   Regardless of the timing method, if delay is greater than two full
+ *   frames (could be greater than 20 or greater than 200), we don't want
+ *   to freeze while waiting for a huge delay. Instead, contine playing 
+ *   video at half speed and continue to read new audio and video frames
+ *   from the file until the sync is 'in the ballpark'.
+ */
 int VideoSync::CalcDelay()
 {
     struct timeval now;
@@ -157,12 +191,6 @@ int VideoSync::CalcDelay()
 
     //cout << "delay " << ret_val << endl;
 
-    // Regardless of the timing method, if delay is greater than two full
-    // frames (could be greater than 20 or greater than 200), we don't want
-    // to freeze while waiting for a huge delay. Instead, contine playing 
-    // video at half speed and continue to read new audio and video frames
-    // from the file until the sync is 'in the ballpark'.
-
     if (ret_val > m_frame_interval * 2)
     {
         if (m_interlaced)
@@ -178,14 +206,16 @@ int VideoSync::CalcDelay()
     return ret_val;
 }
 
+/** \fn VideoSync::KeepPhase()
+ *  \brief Keep our nexttrigger from drifting too close to the exact retrace.
+ *
+ *   If delay is near zero, some frames will be delay < 0 and others
+ *   delay > 0 which would cause continous rapid fire stuttering.
+ *   This method is only useful for those sync methods where WaitForFrame
+ *   targets hardware retrace rather than targeting nexttrigger.
+ */
 void VideoSync::KeepPhase()
 {
-    // Keep our nexttrigger from drifting too close to the exact retrace.
-    // If delay is near zero, some frames will be delay < 0 and others
-    // delay > 0 which would cause continous rapid fire stuttering.
-    // This method is only useful for those sync methods where WaitForFrame
-    // targets hardware retrace rather than targeting nexttrigger.
-
     // cerr << m_delay << endl;
     if (m_delay < -(m_refresh_interval/2))
         OffsetTimeval(m_nexttrigger, 200);
@@ -220,22 +250,26 @@ typedef union drm_wait_vblank {
 
 static int drmWaitVBlank(int fd, drm_wait_vblank_t *vbl)
 {
-    int ret;
+    int ret = -1;
 
+#ifndef _WIN32
     do {
        ret = ioctl(fd, DRM_IOCTL_WAIT_VBLANK, vbl);
        vbl->request.type &= ~DRM_VBLANK_RELATIVE;
     } while (ret && errno == EINTR);
+#endif
 
     return ret;
 }
 
 char *DRMVideoSync::sm_dri_dev = "/dev/dri/card0";
 
-DRMVideoSync::DRMVideoSync(int fr, int ri, bool intl) : VideoSync(fr, ri, intl)
+DRMVideoSync::DRMVideoSync(VideoOutput *vo, int fr, int ri, bool intl) :
+    VideoSync(vo, fr, ri, intl)
 {
     m_dri_fd = -1;
 }
+
 DRMVideoSync::~DRMVideoSync()
 {
     if (m_dri_fd >= 0)
@@ -243,7 +277,7 @@ DRMVideoSync::~DRMVideoSync()
     m_dri_fd = -1;
 }
 
-bool DRMVideoSync::TryInit()
+bool DRMVideoSync::TryInit(void)
 {
     drm_wait_vblank_t blank;
 
@@ -267,7 +301,7 @@ bool DRMVideoSync::TryInit()
     return true;
 }
 
-void DRMVideoSync::Start()
+void DRMVideoSync::Start(void)
 {
     // Wait for a refresh so we start out synched
     drm_wait_vblank_t blank;
@@ -312,7 +346,7 @@ void DRMVideoSync::WaitForFrame(int sync_delay)
     }
 }
 
-void DRMVideoSync::AdvanceTrigger()
+void DRMVideoSync::AdvanceTrigger(void)
 {
     KeepPhase();
     UpdateNexttrigger();
@@ -320,8 +354,11 @@ void DRMVideoSync::AdvanceTrigger()
 
 char *nVidiaVideoSync::sm_nvidia_dev = "/dev/nvidia0";
 
-nVidiaVideoSync::nVidiaVideoSync(int fi, int ri, bool intr) : 
-    VideoSync(fi, ri, intr) {}
+nVidiaVideoSync::nVidiaVideoSync(VideoOutput *vo,
+                                 int fi, int ri, bool intr) : 
+    VideoSync(vo, fi, ri, intr)
+{
+}
 
 nVidiaVideoSync::~nVidiaVideoSync()
 {
@@ -347,7 +384,7 @@ bool nVidiaVideoSync::dopoll() const
     return true;
 }
 
-bool nVidiaVideoSync::TryInit()
+bool nVidiaVideoSync::TryInit(void)
 {
     m_nvidia_fd = open(sm_nvidia_dev, O_RDONLY);
     if (m_nvidia_fd < 0)
@@ -367,7 +404,7 @@ bool nVidiaVideoSync::TryInit()
     return true;
 }
 
-void nVidiaVideoSync::Start()
+void nVidiaVideoSync::Start(void)
 {
     dopoll();
     VideoSync::Start();
@@ -394,53 +431,71 @@ void nVidiaVideoSync::WaitForFrame(int sync_delay)
     }
 }
 
-void nVidiaVideoSync::AdvanceTrigger()
+void nVidiaVideoSync::AdvanceTrigger(void)
 {
     KeepPhase();
     UpdateNexttrigger();
 }
 
-#ifdef USING_OPENGL_VSYNC
-/* Try to create an OpenGL surface so we can use glXWaitVideoSyncSGI:
- * http://osgcvs.no-ip.com/osgarchiver/archives/June2002/0022.html
- * http://www.ac3.edu.au/SGI_Developer/books/OpenGLonSGI/sgi_html/ch10.html#id37188
- * http://www.inb.mu-luebeck.de/~boehme/xvideo_sync.html
- */
-OpenGLVideoSync::OpenGLVideoSync(int fr, int ri, bool intl) : 
-    VideoSync(fr, ri, intl)
+OpenGLVideoSync::OpenGLVideoSync(VideoOutput *video_output,
+                                 int frame_interval, int refresh_interval,
+                                 bool interlaced)
+    : VideoSync(video_output, frame_interval, refresh_interval, interlaced),
+      m_drawable(0), m_context(0)
 {
-    m_display = NULL;
-    m_drawable = 0;
-    m_context = 0;
 }
 
 OpenGLVideoSync::~OpenGLVideoSync()
 {
-    if (m_display != NULL)
-        XCloseDisplay(m_display);
+#ifdef USING_OPENGL_VSYNC
+    VideoOutputXv *vo = dynamic_cast<VideoOutputXv*>(m_video_output);
+    if (vo && vo->XJ_disp)
+    {
+        Stop();
+        if (m_context)
+            X11S(glXDestroyContext(vo->XJ_disp, m_context));
+        if (m_drawable)
+            X11S(XDestroyWindow(vo->XJ_disp, m_drawable));
+    }
+#endif /* USING_OPENGL_VSYNC */
 }
 
-bool OpenGLVideoSync::TryInit()
+/** \fn OpenGLVideoSync::TryInit(void)
+ *  \brief Try to create an OpenGL surface so we can use glXWaitVideoSyncSGI:
+ *  \return true if this method can be employed, false if it can not.
+ */
+bool OpenGLVideoSync::TryInit(void)
 {
-    m_display = XOpenDisplay(NULL);
-    if (m_display == NULL)
+#ifdef USING_OPENGL_VSYNC
+    VideoOutputXv *vo =  dynamic_cast<VideoOutputXv*>(m_video_output);
+    if (!vo)
     {
-        VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: Could not"
-                " open X Window Display.")/*arg(XErrorMessage)*/;
+        VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: No VideoOutput.");
+        return false;
+    }
+
+    if (!vo->XJ_disp)
+    {
+        VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: "
+                "VideoOutputXv does not have X Window Display.");
         return false;
     }
 
     /* Look for GLX at all */
     int ndummy;
-    if (glXQueryExtension(m_display, &ndummy, &ndummy) == 0)
+    int ret;
+    X11S(ret = glXQueryExtension(vo->XJ_disp, &ndummy, &ndummy));
+    if (!ret)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: OpenGL extension not present.");
         return false;
     }
 
     /* Look for video sync extension */
-    const char *xt = glXQueryExtensionsString(m_display, DefaultScreen(m_display));
-    VERBOSE(VB_PLAYBACK, QString("OpenGLVideoSync: GLX extensions: %1").arg(xt));
+    const char *xt = NULL;
+    X11S(xt = glXQueryExtensionsString(vo->XJ_disp, DefaultScreen(vo->XJ_disp)));
+    VERBOSE(VB_PLAYBACK, QString("OpenGLVideoSync: GLX extensions: %1")
+            .arg(xt));
     if (strstr(xt, "GLX_SGI_video_sync") == NULL)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: GLX Video Sync"
@@ -456,42 +511,59 @@ bool OpenGLVideoSync::TryInit()
     XVisualInfo *vis;
     XSetWindowAttributes swa;
     Window w;
-    
-    vis = glXChooseVisual(m_display, DefaultScreen(m_display), attribList);
+
+    X11S(vis = glXChooseVisual(vo->XJ_disp, vo->XJ_screen_num, attribList));
     if (vis == NULL) 
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: No appropriate visual found");
         return false;
     }
-    swa.colormap = XCreateColormap(m_display, RootWindow(m_display, vis->screen),
-                                   vis->visual, AllocNone);
+    X11S(swa.colormap = XCreateColormap(vo->XJ_disp,
+                                        RootWindow(vo->XJ_disp, vis->screen),
+                                        vis->visual, AllocNone));
     if (swa.colormap == 0)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: Failed to create colormap");
         return false;
     }
-    w = XCreateWindow(m_display, RootWindow(m_display, vis->screen), 0, 0, 1, 1,
-                      0, vis->depth, InputOutput, vis->visual, 
-                      CWColormap, &swa);
+
+    MythMainWindow *mainWindow = gContext->GetMainWindow();
+    uint x = mainWindow->x() + (mainWindow->width()/2);
+    uint y = mainWindow->y() + (mainWindow->height()/2);
+    VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: x,y -> "<<x<<", "<<y);
+    X11S(w = XCreateWindow(vo->XJ_disp, RootWindow(vo->XJ_disp, vis->screen),
+                           x, y, 1 /*width*/, 1 /*height*/,
+                           0 /*border width*/, vis->depth, InputOutput,
+                           vis->visual, CWColormap, &swa));
     if (w == 0)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: Failed to create dummy window");
         return false;
     }
-    m_context = glXCreateContext(m_display, vis, None, GL_TRUE);
+    m_drawable = w;
+
+    X11S(m_context = glXCreateContext(vo->XJ_disp, vis, None, GL_TRUE));
     if (m_context == NULL)
     {
         VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: Failed to create GLX context");
         return false;
     }
-    m_drawable = w;
-    if (glXMakeCurrent(m_display, m_drawable, m_context) != False)
+    X11S(ret = glXMakeCurrent(vo->XJ_disp, m_drawable, m_context));
+    if (ret != False)
     {
         unsigned int count;
-        int ok = glXGetVideoSyncSGI(&count);
-        if (GLX_BAD_CONTEXT == ok)
+        X11S(ret = glXGetVideoSyncSGI(&count));
+        if (ret == 0)
+        {
+            VERBOSE(VB_PLAYBACK, "Using OpenGLVideoSync");
+            return true;
+        }
+        else if (GLX_BAD_CONTEXT == ret)
             VERBOSE(VB_PLAYBACK, "OpenGLVideoSync: Bad Context for VSync");
-        return GLX_BAD_CONTEXT != ok;
+        else
+            VERBOSE(VB_PLAYBACK, QString("OpenGLVideoSync: Could not get "
+                                         "VSync support ret=%1").arg(ret));
+        return false;
     }
     else
     {
@@ -499,71 +571,131 @@ bool OpenGLVideoSync::TryInit()
                 "OpenGLVideoSync: Failed to make GLX context current context");
         return false;
     }
+#else
+    return false;
+#endif /* USING_OPENGL_VSYNC */
 }
 
-void OpenGLVideoSync::Start()
+/** \fn checkGLSyncError(const QString&, int)
+ *  \brief Prints an error messages for SGI_video_sync extension calls.
+ *
+ *  \return true if all is ok, false if there is an error
+ */
+bool checkGLSyncError(const QString& hdr, int err)
 {
+    (void) hdr;
+    (void) err;
+#ifdef USING_OPENGL_VSYNC
+    QString errStr("");
+    switch (err)
+    {
+        case False:
+            break;
+        case GLX_BAD_CONTEXT:
+            errStr = "Bad Context";
+            break;
+        case GLX_BAD_VALUE:
+            errStr = "Bad Value";
+            break;
+        default:
+            errStr = QString("Unknown Error 0x%1").arg(err,0,16);
+            break;
+    }
+    if (errStr != "")
+    {
+        VERBOSE(VB_IMPORTANT, hdr+" reported error: "+errStr);
+        return false;
+    }
+    return true;
+#else
+    return false;
+#endif /* USING_OPENGL_VSYNC */
+}
+
+void OpenGLVideoSync::Start(void)
+{
+#ifdef USING_OPENGL_VSYNC
+    VideoOutputXv *vo = dynamic_cast<VideoOutputXv*>(m_video_output);
+    if (!vo || !vo->XJ_disp)
+        return;
+
+    int err;
+    // Bind OpenGL context to current thread.
+    X11S(err = glXMakeCurrent(vo->XJ_disp, m_drawable, m_context));
+    if (err != True)
+    {
+        VERBOSE(VB_PLAYBACK,
+                "OpenGLVideoSync::Start(): Failed to make GLX context "
+                "current context. A/V Sync will probably fail.");
+    }
+
     // Wait for a refresh so we start out synched
     unsigned int count;
-    int r;
-    r = glXMakeCurrent(m_display, m_drawable, m_context);
-    r = glXGetVideoSyncSGI(&count);
-    r = glXWaitVideoSyncSGI(2, (count+1)%2 ,&count);
+    err = glXGetVideoSyncSGI(&count);
+    checkGLSyncError("OpenGLVideoSync::Start(): Frame Number Query", err);
+    err = glXWaitVideoSyncSGI(2, (count+1)%2 ,&count);
+    checkGLSyncError("OpenGLVideoSync::Start(): A/V Sync", err);
+
+    // Initialize next trigger 
     VideoSync::Start();
+#endif /* USING_OPENGL_VSYNC */
 }
 
 void OpenGLVideoSync::WaitForFrame(int sync_delay)
 {
-    unsigned int count, n;
-    int r;
-
+    (void) sync_delay;
+#ifdef USING_OPENGL_VSYNC
+    const QString msg1("First A/V Sync"), msg2("Second A/V Sync");
     OffsetTimeval(m_nexttrigger, sync_delay);
 
-    m_delay = CalcDelay();
-    //cerr << "WaitForFrame at : " << m_delay;
+    unsigned int frameNum = 0;
+    int err = glXGetVideoSyncSGI(&frameNum);
+    checkGLSyncError("Frame Number Query", err);
 
     // Always sync to the next retrace execpt when we are very late.
-    if (m_delay > -(m_refresh_interval/2)) 
+    if ((m_delay = CalcDelay()) > -(m_refresh_interval/2)) 
     {
-        r = glXMakeCurrent(m_display, m_drawable, m_context);
-        r = glXGetVideoSyncSGI(&count);
-        r = glXWaitVideoSyncSGI(2, (count+1)%2 ,&count);
+        err = glXWaitVideoSyncSGI(2, (frameNum+1)%2 ,&frameNum);
+        checkGLSyncError(msg1, err);
         m_delay = CalcDelay();
-        //cerr << "\tDelay at sync: " << m_delay;
     }
-    //cerr << endl;
 
+    // Wait for any remaining retrace intervals in one pass.
     if (m_delay > 0)
     {
-        // Wait for any remaining retrace intervals in one pass.
-        n = m_delay / m_refresh_interval + 1;
-        r = glXWaitVideoSyncSGI((n+1), (count+n)%(n+1), &count);
+        uint n = m_delay / m_refresh_interval + 1;
+        err = glXWaitVideoSyncSGI((n+1), (frameNum+n)%(n+1), &frameNum);
+        checkGLSyncError(msg2, err);
         m_delay = CalcDelay();
-        //cerr << "Wait " << n << " intervals. Count " << count;
-        //cerr  << " Delay " << m_delay << endl;
     }
+#endif /* USING_OPENGL_VSYNC */
 }
 
-void OpenGLVideoSync::AdvanceTrigger()
+void OpenGLVideoSync::AdvanceTrigger(void)
 {
+#ifdef USING_OPENGL_VSYNC
     KeepPhase();
     UpdateNexttrigger();
+#endif /* USING_OPENGL_VSYNC */
 }
 
-void OpenGLVideoSync::Stop()
+/** \fn OpenGLVideoSync::Stop(void)
+ *  \brief Stops VSync; must be called from main thread.
+ *
+ */
+void OpenGLVideoSync::Stop(void)
 {
-    // This method should be called from the MAIN THREAD to work around
-    // a bug in some OpenGL implementations, where if the thread that
-    // has the current GL context goes away, no other thread can ever
-    // have it.
-    glXMakeCurrent(m_display, 0, 0);
-}
+#ifdef USING_OPENGL_VSYNC
+    VideoOutputXv *vo = dynamic_cast<VideoOutputXv*>(m_video_output);
+    if (vo && vo->XJ_disp)
+        X11S(glXMakeCurrent(vo->XJ_disp, 0, 0));
 #endif /* USING_OPENGL_VSYNC */
+}
 
 #ifdef __linux__
 #define RTCRATE 1024
-RTCVideoSync::RTCVideoSync(int fi, int ri, bool intr) :
-    VideoSync(fi, ri, intr)
+RTCVideoSync::RTCVideoSync(VideoOutput *vo, int fi, int ri, bool intr) :
+    VideoSync(vo, fi, ri, intr)
 {
     m_rtcfd = -1;
 }
@@ -574,7 +706,7 @@ RTCVideoSync::~RTCVideoSync()
         close(m_rtcfd);
 }
 
-bool RTCVideoSync::TryInit()
+bool RTCVideoSync::TryInit(void)
 {
     m_rtcfd = open("/dev/rtc", O_RDONLY);
     if (m_rtcfd < 0)
@@ -616,14 +748,15 @@ void RTCVideoSync::WaitForFrame(int sync_delay)
     }
 }
 
-void RTCVideoSync::AdvanceTrigger()
+void RTCVideoSync::AdvanceTrigger(void)
 {
     UpdateNexttrigger();
 }
 #endif /* __linux__ */
 
-BusyWaitVideoSync::BusyWaitVideoSync(int fr, int ri, bool intl) : 
-    VideoSync(fr, ri, intl) 
+BusyWaitVideoSync::BusyWaitVideoSync(VideoOutput *vo,
+                                     int fr, int ri, bool intl) : 
+    VideoSync(vo, fr, ri, intl) 
 {
     m_cheat = 5000;
     m_fudge = 0;
@@ -633,7 +766,7 @@ BusyWaitVideoSync::~BusyWaitVideoSync()
 {
 }
 
-bool BusyWaitVideoSync::TryInit()
+bool BusyWaitVideoSync::TryInit(void)
 {
     return true;
 }
@@ -668,13 +801,14 @@ void BusyWaitVideoSync::WaitForFrame(int sync_delay)
     }
 }
 
-void BusyWaitVideoSync::AdvanceTrigger()
+void BusyWaitVideoSync::AdvanceTrigger(void)
 {
     UpdateNexttrigger();
 }
 
-USleepVideoSync::USleepVideoSync(int fr, int ri, bool intl) : 
-    VideoSync(fr, ri, intl) 
+USleepVideoSync::USleepVideoSync(VideoOutput *vo,
+                                 int fr, int ri, bool intl) : 
+    VideoSync(vo, fr, ri, intl)
 {
 }
 
@@ -682,7 +816,7 @@ USleepVideoSync::~USleepVideoSync()
 {
 }
 
-bool USleepVideoSync::TryInit()
+bool USleepVideoSync::TryInit(void)
 {
     return true;
 }
@@ -697,7 +831,7 @@ void USleepVideoSync::WaitForFrame(int sync_delay)
         usleep(m_delay);
 }
 
-void USleepVideoSync::AdvanceTrigger()
+void USleepVideoSync::AdvanceTrigger(void)
 {
     UpdateNexttrigger();
 }
