@@ -1,7 +1,6 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <signal.h>
 #include <unistd.h>
 #include <qsqldatabase.h>
 #include <qsqlquery.h>
@@ -17,14 +16,9 @@ using namespace std;
 
 #include "libmyth/mythcontext.h"
 #include "libmyth/mythdbcon.h"
+#include "libmyth/util.h"
 
-bool HouseKeeper_filldb_running = false;
-
-void reapChild(int /* sig */)
-{
-    (void)wait(0);
-    HouseKeeper_filldb_running = false;
-}
+static bool HouseKeeper_filldb_running = false;
 
 HouseKeeper::HouseKeeper(bool runthread, bool master)
 {
@@ -33,6 +27,8 @@ HouseKeeper::HouseKeeper(bool runthread, bool master)
     threadrunning = runthread;
     filldbRunning = false;
 
+    CleanupMyOldRecordings();
+
     if (runthread)
     {
         pthread_t hkthread;
@@ -49,6 +45,12 @@ bool HouseKeeper::wantToRun(const QString &dbTag, int period, int minhour,
 {
     bool runOK = false;
     unsigned int oneday = 60 * 60 * 24;
+    unsigned int longEnough = 0;
+
+    if (period)
+        longEnough = ((period * oneday) - 600);
+    else
+        longEnough = oneday / 8;
 
     QDateTime now = QDateTime::currentDateTime();
     QDateTime lastrun;
@@ -57,16 +59,15 @@ bool HouseKeeper::wantToRun(const QString &dbTag, int period, int minhour,
     MSqlQuery result(MSqlQuery::InitCon());
     if (result.isConnected())
     {
-        QString query = QString("SELECT lastrun FROM housekeeping WHERE "
-                                "tag = \"%1\";").arg(dbTag);
-        result.prepare(query);
+        result.prepare("SELECT lastrun FROM housekeeping WHERE tag = :TAG ;");
+        result.bindValue(":TAG", dbTag);
 
         if (result.exec() && result.isActive() && result.size() > 0)
         {
             result.next();
             lastrun = result.value(0).toDateTime();
 
-            if ((now.toTime_t() - lastrun.toTime_t()) > period * oneday)
+            if ((now.toTime_t() - lastrun.toTime_t()) > longEnough)
             {
                 int hour = now.toString(QString("h")).toInt();
                 if ((hour >= minhour) && (hour <= maxhour))
@@ -75,9 +76,9 @@ bool HouseKeeper::wantToRun(const QString &dbTag, int period, int minhour,
         }
         else
         {
-            query = QString("INSERT INTO housekeeping(tag,lastrun) "
-                            "values(\"%1\",now())").arg(dbTag);
-            result.prepare(query);
+            result.prepare("INSERT INTO housekeeping(tag,lastrun) "
+                           "values(:TAG ,now());");
+            result.bindValue(":TAG", dbTag);
             result.exec();
 
             runOK = true;
@@ -92,17 +93,36 @@ void HouseKeeper::updateLastrun(const QString &dbTag)
     MSqlQuery result(MSqlQuery::InitCon());
     if (result.isConnected())
     {
-        QString query = QString("DELETE FROM housekeeping WHERE "
-                                "tag = \"%1\";").arg(dbTag);
-        result.prepare(query);
+        result.prepare("DELETE FROM housekeeping WHERE tag = :TAG ;");
+        result.bindValue(":TAG", dbTag);
         result.exec();
-        query = QString("INSERT INTO housekeeping(tag,lastrun) "
-                        "values(\"%1\",now())").arg(dbTag);
-        result.prepare(query);
+
+        result.prepare("INSERT INTO housekeeping(tag,lastrun) "
+                       "values(:TAG ,now()) ;");
+        result.bindValue(":TAG", dbTag);
         result.exec();
     }
 }
 
+QDateTime HouseKeeper::getLastRun(const QString &dbTag)
+{
+    QDateTime lastRun;
+    MSqlQuery result(MSqlQuery::InitCon());
+
+    lastRun.setTime_t(0);
+
+    result.prepare("SELECT lastrun FROM housekeeping WHERE tag = :TAG ;");
+    result.bindValue(":TAG", dbTag);
+
+    if (result.exec() && result.isActive() && result.size() > 0)
+    {
+        result.next();
+        lastRun = result.value(0).toDateTime();
+    }
+
+    return lastRun;
+}
+
 void HouseKeeper::RunHouseKeeping(void)
 {
     int period, maxhr, minhr;
@@ -153,7 +173,44 @@ void HouseKeeper::RunHouseKeeping(void)
                         maxhr = gContext->GetNumSetting("MythFillMaxHour", 24);
                     }
 
-                    if (wantToRun("MythFillDB", period, minhr, maxhr))
+                    bool grabberSupportsNextTime = false;
+                    MSqlQuery result(MSqlQuery::InitCon());
+                    if (result.isConnected())
+                    {
+                        result.prepare("SELECT COUNT(*) FROM videosource "
+                                       "WHERE xmltvgrabber IN "
+                                           "( 'datadirect', 'technovera' );");
+
+                        if ((result.exec()) &&
+                            (result.isActive()) &&
+                            (result.size() > 0) &&
+                            (result.next()) &&
+                            (result.value(0).toInt() > 0))
+                            grabberSupportsNextTime = true;
+                    }
+
+                    bool runMythFill = false;
+                    if (grabberSupportsNextTime &&
+                        gContext->GetNumSetting("MythFillGrabberSuggestsTime", 1))
+                    {
+                        QDateTime nextRun = QDateTime::fromString(
+                            gContext->GetSetting("MythFillSuggestedRunTime",
+                            "1970-01-01T00:00:00"), Qt::ISODate);
+                        QDateTime lastRun = getLastRun("MythFillDB");
+                        QDateTime now = QDateTime::currentDateTime();
+                        int hour = now.toString(QString("h")).toInt();
+
+                        if ((nextRun < now) &&
+                            (lastRun.secsTo(now) > (3 * 60 * 60)) &&
+                            ((minhr <= hour) && (hour <= maxhr)))
+                            runMythFill = true;
+                    }
+                    else if (wantToRun("MythFillDB", period, minhr, maxhr))
+                    {
+                        runMythFill = true;
+                    }
+
+                    if (runMythFill)
                     {
                         QString msg = "Running mythfilldatabase";
                         gContext->LogEntry("mythbackend", LP_DEBUG, msg, "");
@@ -165,12 +222,24 @@ void HouseKeeper::RunHouseKeeping(void)
             }
 
             if (wantToRun("JobQueueCleanup", 1, 0, 24))
+            {
                 JobQueue::CleanupOldJobsInQueue();
+                updateLastrun("JobQueueCleanup");
+            }
+
+            if (wantToRun("InUseProgramsCleanup", 1, 0, 24))
+            {
+                CleanupAllOldInUsePrograms();
+                updateLastrun("InUseProgramsCleanup");
+            }
         }
 
         dbTag = QString("JobQueueRecover-%1").arg(gContext->GetHostName());
         if (wantToRun(dbTag, 1, 0, 24))
+        {
             JobQueue::RecoverOldJobsInQueue();
+            updateLastrun(dbTag);
+        }
 
         sleep(300);
     }
@@ -189,22 +258,25 @@ void HouseKeeper::flushLogs()
     MSqlQuery result(MSqlQuery::InitCon());
     if (result.isConnected())
     {
-        QString dstring = days.toString(QString("yyyy-MM-dd hh:mm:ss"));
-        QString query = QString("DELETE FROM mythlog WHERE "
-                                "acknowledged=1 and logdate<\"%1\";")
-                                .arg(dstring);
-        result.prepare(query);
+        result.prepare("DELETE FROM mythlog WHERE "
+                       "acknowledged=1 and logdate < :DAYS ;");
+        result.bindValue(":DAYS", days);
         result.exec();
 
-        dstring = max.toString(QString("yyyy-MM-dd hh:mm:ss"));
-        query = QString("DELETE FROM mythlog WHERE logdate<\"%1\";")
-                                .arg(dstring);
-        result.prepare(query);
+        result.prepare("DELETE FROM mythlog WHERE logdate< :MAX ;");
+        result.bindValue(":MAX", max);
         result.exec();
     }
 }
 
-void HouseKeeper::runFillDatabase()
+void *HouseKeeper::runMFDThread(void *param)
+{
+    HouseKeeper *keep = (HouseKeeper *)param;
+    keep->RunMFD();
+    return NULL;
+}
+
+void HouseKeeper::RunMFD(void)
 {
     QString command;
 
@@ -218,16 +290,45 @@ void HouseKeeper::runFillDatabase()
         command = QString("%1 %2").arg(mfpath).arg(mfarg);
     else
         command = QString("%1 %2 >>%3 2>&1").arg(mfpath).arg(mfarg).arg(mflog);
-    
-    signal(SIGCHLD, &reapChild);
+
+    myth_system(command.ascii(), MYTH_SYSTEM_DONT_BLOCK_LIRC | 
+                                 MYTH_SYSTEM_DONT_BLOCK_JOYSTICK_MENU);
+
+    HouseKeeper_filldb_running = false;
+}
+
+void HouseKeeper::runFillDatabase()
+{
+    if (HouseKeeper_filldb_running)
+        return;
+
     HouseKeeper_filldb_running = true;
-    if (fork() == 0)
-    {
-        for(int i = 3; i < sysconf(_SC_OPEN_MAX) - 1; ++i)
-            close(i);
-        system(command.ascii());
-        _exit(0);
-    }
+
+    pthread_t housekeep_thread;
+    pthread_create(&housekeep_thread, NULL, runMFDThread, this);
+    pthread_detach(housekeep_thread);
+}
+
+void HouseKeeper::CleanupMyOldRecordings(void)
+{
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    query.prepare("DELETE FROM inuseprograms "
+                  "WHERE hostname = :HOSTNAME AND "
+                    "( recusage = 'recorder' OR recusage LIKE 'Unknown %' );");
+    query.bindValue(":HOSTNAME", gContext->GetHostName());
+    query.exec();
+}
+
+void HouseKeeper::CleanupAllOldInUsePrograms(void)
+{
+    QDateTime fourHoursAgo = QDateTime::currentDateTime().addSecs(-4 * 60 * 60);
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    query.prepare("DELETE FROM inuseprograms "
+                  "WHERE lastupdatetime < :FOURHOURSAGO ;");
+    query.bindValue(":FOURHOURSAGO", fourHoursAgo);
+    query.exec();
 }
 
 void *HouseKeeper::doHouseKeepingThread(void *param)
@@ -237,3 +338,5 @@ void *HouseKeeper::doHouseKeepingThread(void *param)
  
     return NULL;
 }
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
