@@ -2,18 +2,25 @@
 #include <qsqldatabase.h>
 #include <qfile.h>
 #include <qmap.h>
+#include <qregexp.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <libgen.h>
 #include <signal.h>
+#include <cerrno>
+
+#include "mythconfig.h"
+#ifdef CONFIG_DARWIN
+    #include <sys/aio.h>    // O_SYNC
+#endif
 
 #include <iostream>
 #include <fstream>
 using namespace std;
 
-#include "tv.h"
+#include "tv_rec.h"
 #include "autoexpire.h"
 #include "scheduler.h"
 #include "mainserver.h"
@@ -21,10 +28,17 @@ using namespace std;
 #include "remoteutil.h"
 #include "housekeeper.h"
 
-#include "libmythtv/programinfo.h"
 #include "libmyth/mythcontext.h"
+#include "libmyth/mythdbcon.h"
+#include "libmyth/exitcodes.h"
+#include "libmythtv/programinfo.h"
 #include "libmythtv/dbcheck.h"
 #include "libmythtv/jobqueue.h"
+#include "libmythupnp/upnp.h"
+
+#include "upnpcdstv.h"
+#include "upnpcdsmusic.h"
+#include "httpstatus.h"
 
 QMap<int, EncoderLink *> tvList;
 AutoExpire *expirer = NULL;
@@ -33,15 +47,30 @@ JobQueue *jobqueue = NULL;
 QString pidfile;
 QString lockfile_location;
 HouseKeeper *housekeeping = NULL;
+QString logfile = "";
 
-bool setupTVs(bool ismaster)
+HttpServer *g_pHttpServer = NULL;
+UPnp       *g_pUPnp       = NULL;
+
+bool setupTVs(bool ismaster, bool &error)
 {
+    error = false;
     QString localhostname = gContext->GetHostName();
 
-    QSqlQuery query;
+    MSqlQuery query(MSqlQuery::InitCon());
 
     if (ismaster)
     {
+        // Hack to make sure recorded.basename gets set if the user
+        // downgrades to a prior version and creates new entries
+        // without it.
+        if (!query.exec("UPDATE recorded SET basename = CONCAT(chanid, '_', "
+                        "DATE_FORMAT(starttime, '%Y%m%d%H%i00'), '_', "
+                        "DATE_FORMAT(endtime, '%Y%m%d%H%i00'), '.nuv') "
+                        "WHERE basename = '';"))
+            MythContext::DBError("Updating record basename",
+                                 query.lastQuery());
+
         // Hack to make sure record.station gets set if the user
         // downgrades to a prior version and creates new entries
         // without it.
@@ -63,12 +92,14 @@ bool setupTVs(bool ismaster)
             MythContext::DBError("Querying minimum chanid",
                                  query.lastQuery());
 
-        QSqlQuery records_without_station("SELECT record.chanid,"
+        MSqlQuery records_without_station(MSqlQuery::InitCon());
+        records_without_station.prepare("SELECT record.chanid,"
                 " channel.callsign FROM record LEFT JOIN channel"
                 " ON record.chanid = channel.chanid WHERE record.station='';");
+        records_without_station.exec(); 
         if (records_without_station.first())
         {
-            QSqlQuery update_record;
+            MSqlQuery update_record(MSqlQuery::InitCon());
             update_record.prepare("UPDATE record SET station = :CALLSIGN"
                     " WHERE chanid = :CHANID;");
             do
@@ -86,57 +117,79 @@ bool setupTVs(bool ismaster)
         }
     }
 
-    query.exec("SELECT cardid,hostname FROM capturecard ORDER BY cardid;");
+    query.exec("SELECT cardid, hostname "
+               "FROM capturecard "
+               "WHERE parentid = '0' "
+               "ORDER BY cardid");
 
-    if (query.isActive() && query.numRowsAffected())
+    while (query.isActive() && query.next())
     {
-        while (query.next())
+        uint    cardid = query.value(0).toUInt();
+        QString host   = query.value(1).toString();
+        QString cidmsg = QString("Card %1").arg(cardid);
+
+        if (host.isEmpty())
         {
-            int cardid = query.value(0).toInt();
-            QString host = query.value(1).toString();
+            QString msg = cidmsg + " does not have a hostname defined.\n"
+                "Please run setup and confirm all of the capture cards.\n";
 
-            if (host.isNull() || host.isEmpty())
+            cerr << msg;
+            gContext->LogEntry("mythbackend", LP_CRITICAL,
+                               "Problem with capture cards", msg);
+            continue;
+        }
+
+        if (!ismaster)
+        {
+            if (host == localhostname)
             {
-                QString msg = "One of your capturecard entries does not have a "
-                              "hostname.\n  Please run setup and confirm all "
-                              "of the capture cards.\n";
-
-                cerr << msg;
-                gContext->LogEntry("mythbackend", LP_CRITICAL,
-                                   "Problem with capture cards", msg);
-                exit(2);
-            }
-
-            if (!ismaster)
-            {
-                if (host == localhostname)
+                TVRec *tv = new TVRec(cardid);
+                if (tv->Init())
                 {
-                    TVRec *tv = new TVRec(cardid);
-                    tv->Init();
-                    EncoderLink *enc = new EncoderLink(cardid, tv);
-                    tvList[cardid] = enc;
-                }
-            }
-            else
-            {
-                if (host == localhostname)
-                {
-                    TVRec *tv = new TVRec(cardid);
-                    tv->Init();
                     EncoderLink *enc = new EncoderLink(cardid, tv);
                     tvList[cardid] = enc;
                 }
                 else
                 {
-                    EncoderLink *enc = new EncoderLink(cardid, NULL, host);
-                    tvList[cardid] = enc;
+                    gContext->LogEntry("mythbackend", LP_CRITICAL,
+                                       "Problem with capture cards",
+                                       cidmsg + "failed init");
+                    delete tv;
+                    // The master assumes card comes up so we need to
+                    // set error and exit if a non-master card fails.
+                    error = true;
                 }
             }
         }
+        else
+        {
+            if (host == localhostname)
+            {
+                TVRec *tv = new TVRec(cardid);
+                if (tv->Init())
+                {
+                    EncoderLink *enc = new EncoderLink(cardid, tv);
+                    tvList[cardid] = enc;
+                }
+                else
+                {
+                    gContext->LogEntry("mythbackend", LP_CRITICAL,
+                                       "Problem with capture cards",
+                                       cidmsg + "failed init");
+                    delete tv;
+                }
+            }
+            else
+            {
+                EncoderLink *enc = new EncoderLink(cardid, NULL, host);
+                tvList[cardid] = enc;
+            }
+        }
     }
-    else
+
+    if (tvList.empty())
     {
-        cerr << "ERROR: no capture cards are defined in the database.\n";
+        cerr << "ERROR: no valid capture cards are defined in the database.\n";
         cerr << "Perhaps you should read the installation instructions?\n";
         gContext->LogEntry("mythbackend", LP_CRITICAL,
                            "No capture cards are defined", 
@@ -154,12 +207,52 @@ void cleanup(void)
     if (sched)
         delete sched;
 
+    if (g_pUPnp)
+        delete g_pUPnp;
+
+    if (g_pHttpServer)
+        delete g_pHttpServer;
+
     if (pidfile != "")
         unlink(pidfile.ascii());
 
     unlink(lockfile_location.ascii());
+
+    signal(SIGHUP, SIG_DFL);
 }
-    
+
+int log_rotate(int report_error)
+{
+    /* http://www.gossamer-threads.com/lists/mythtv/dev/110113 */
+
+    int new_logfd = open(logfile, O_WRONLY|O_CREAT|O_APPEND|O_SYNC, 0664);
+    if (new_logfd < 0)
+    {
+        // If we can't open the new logfile, send data to /dev/null
+        if (report_error)
+        {
+            cerr << "cannot open logfile " << logfile << endl;
+            return -1;
+        }
+        new_logfd = open("/dev/null", O_WRONLY);
+        if (new_logfd < 0)
+        {
+            // There's not much we can do, so punt.
+            return -1;
+        }
+    }
+    while (dup2(new_logfd, 1) < 0 && errno == EINTR) ;
+    while (dup2(new_logfd, 2) < 0 && errno == EINTR) ;
+    while (close(new_logfd) < 0 && errno == EINTR) ;
+    return 0;
+}
+
+void log_rotate_handler(int)
+{
+    log_rotate(0);
+}
+
+
 int main(int argc, char **argv)
 {
     for(int i = 3; i < sysconf(_SC_OPEN_MAX) - 1; ++i)
@@ -167,15 +260,17 @@ int main(int argc, char **argv)
 
     QApplication a(argc, argv, false);
 
-    QString logfile = "";
+    QMap<QString, QString> settingsOverride;
     QString binname = basename(a.argv()[0]);
-    QString verboseString = QString(" important general");
 
     bool daemonize = false;
     bool printsched = false;
     bool testsched = false;
     bool resched = false;
     bool nosched = false;
+    bool nojobqueue = false;
+    bool nohousekeeper = false;
+    bool noexpirer = false;
     bool printexpire = false;
     for (int argpos = 1; argpos < a.argc(); ++argpos)
     {
@@ -188,7 +283,7 @@ int main(int argc, char **argv)
                 if (logfile.startsWith("-"))
                 {
                     cerr << "Invalid or missing argument to -l/--logfile option\n";
-                    return -1;
+                    return BACKEND_EXIT_INVALID_CMDLINE;
                 }
                 else
                 {
@@ -205,7 +300,7 @@ int main(int argc, char **argv)
                 if (pidfile.startsWith("-"))
                 {
                     cerr << "Invalid or missing argument to -p/--pidfile option\n";
-                    return -1;
+                    return BACKEND_EXIT_INVALID_CMDLINE;
                 } 
                 else
                 {
@@ -219,112 +314,52 @@ int main(int argc, char **argv)
             daemonize = true;
 
         }
+        else if (!strcmp(a.argv()[argpos],"-O") ||
+                 !strcmp(a.argv()[argpos],"--override-setting"))
+        {
+            if (a.argc()-1 > argpos)
+            {
+                QString tmpArg = a.argv()[argpos+1];
+                if (tmpArg.startsWith("-"))
+                {
+                    cerr << "Invalid or missing argument to -O/--override-setting option\n";
+                    return BACKEND_EXIT_INVALID_CMDLINE;
+                } 
+ 
+                QStringList pairs = QStringList::split(",", tmpArg);
+                for (unsigned int index = 0; index < pairs.size(); ++index)
+                {
+                    QStringList tokens = QStringList::split("=", pairs[index]);
+                    tokens[0].replace(QRegExp("^[\"']"), "");
+                    tokens[0].replace(QRegExp("[\"']$"), "");
+                    tokens[1].replace(QRegExp("^[\"']"), "");
+                    tokens[1].replace(QRegExp("[\"']$"), "");
+                    settingsOverride[tokens[0]] = tokens[1];
+                }
+            }
+            else
+            {
+                cerr << "Invalid or missing argument to -O/--override-setting option\n";
+                return BACKEND_EXIT_INVALID_CMDLINE;
+            }
+
+            ++argpos;
+        }
         else if (!strcmp(a.argv()[argpos],"-v") ||
                  !strcmp(a.argv()[argpos],"--verbose"))
         {
-            if (a.argc() > argpos)
+            if (a.argc()-1 > argpos)
             {
-                QString temp = a.argv()[argpos+1];
-                if (temp.startsWith("-"))
-                {
-                    cerr << "Invalid or missing argument to -v/--verbose option\n";
-                    return -1;
-                } 
-                else
-                {
-                    QStringList verboseOpts;
-                    verboseOpts = QStringList::split(',',a.argv()[argpos+1]);
-                    ++argpos;
-                    for (QStringList::Iterator it = verboseOpts.begin(); 
-                         it != verboseOpts.end(); ++it )
-                    {
-                        if(!strcmp(*it,"none"))
-                        {
-                            print_verbose_messages = VB_NONE;
-                            verboseString = "";
-                        }
-                        else if(!strcmp(*it,"all"))
-                        {
-                            print_verbose_messages = VB_ALL;
-                            verboseString = "all";
-                        }
-                        else if(!strcmp(*it,"quiet"))
-                        {
-                            print_verbose_messages = VB_IMPORTANT;
-                            verboseString = "important";
-                        }
-                        else if(!strcmp(*it,"record"))
-                        {
-                            print_verbose_messages |= VB_RECORD;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"playback"))
-                        {
-                            print_verbose_messages |= VB_PLAYBACK;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"channel"))
-                        {
-                            print_verbose_messages |= VB_CHANNEL;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"osd"))
-                        {
-                            print_verbose_messages |= VB_OSD;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"file"))
-                        {
-                            print_verbose_messages |= VB_FILE;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"schedule"))
-                        {
-                            print_verbose_messages |= VB_SCHEDULE;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"network"))
-                        {
-                            print_verbose_messages |= VB_NETWORK;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"commflag"))
-                        {
-                            print_verbose_messages |= VB_COMMFLAG;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"jobqueue"))
-                        {
-                            print_verbose_messages |= VB_JOBQUEUE;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"audio"))
-                        {
-                            print_verbose_messages |= VB_AUDIO;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"libav"))
-                        {
-                            print_verbose_messages |= VB_LIBAV;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"siparser"))
-                        {
-                            print_verbose_messages |= VB_SIPARSER;
-                            verboseString += " " + *it;
-                        }
-                        else
-                        {
-                            cerr << "Unknown argument for -v/--verbose: "
-                                 << *it << endl;;
-                        }
-                    }
-                }
+                if (parse_verbose_arg(a.argv()[argpos+1]) ==
+                        GENERIC_EXIT_INVALID_CMDLINE)
+                    return BACKEND_EXIT_INVALID_CMDLINE;
+
+                ++argpos;
             } 
             else
             {
                 cerr << "Missing argument to -v/--verbose option\n";
-                return -1;
+                return BACKEND_EXIT_INVALID_CMDLINE;
             }
         } 
         else if (!strcmp(a.argv()[argpos],"--printsched"))
@@ -343,14 +378,32 @@ int main(int argc, char **argv)
         {
             nosched = true;
         } 
+        else if (!strcmp(a.argv()[argpos],"--nojobqueue"))
+        {
+            nojobqueue = true;
+        } 
+        else if (!strcmp(a.argv()[argpos],"--nohousekeeper"))
+        {
+            nohousekeeper = true;
+        } 
+        else if (!strcmp(a.argv()[argpos],"--noautoexpire"))
+        {
+            noexpirer = true;
+        } 
         else if (!strcmp(a.argv()[argpos],"--printexpire"))
         {
             printexpire = true;
         } 
         else if (!strcmp(a.argv()[argpos],"--version"))
         {
-            cout << MYTH_BINARY_VERSION << endl;
-            exit(0);
+            extern const char *myth_source_version;
+            cout << "Library API version: " << MYTH_BINARY_VERSION << endl;
+            cout << "Source code version: " << myth_source_version << endl;
+#ifdef MYTH_BUILD_CONFIG
+            cout << "Options compiled in:" <<endl;
+            cout << MYTH_BUILD_CONFIG << endl;
+#endif
+            return BACKEND_EXIT_OK;
         }
         else
         {
@@ -358,35 +411,31 @@ int main(int argc, char **argv)
                 !strcmp(a.argv()[argpos],"--help")))
                 cerr << "Invalid argument: " << a.argv()[argpos] << endl;
             cerr << "Valid options are: " << endl <<
+                    "-h or --help                   List valid command line parameters" << endl <<
                     "-l or --logfile filename       Writes STDERR and STDOUT messages to filename" << endl <<
                     "-p or --pidfile filename       Write PID of mythbackend " <<
                                                     "to filename" << endl <<
                     "-d or --daemon                 Runs mythbackend as a daemon" << endl <<
-                    "-v or --verbose debug-level    Prints more information" << endl <<
-                    "                               Accepts any combination (separated by comma)" << endl << 
-                    "                               of all,none,quiet,record,playback,channel," << endl <<
-                    "                               osd,file,schedule,network,commflag,audio," << endl <<
-                    "                               libav,jobqueue" << endl <<
+                    "-v or --verbose debug-level    Use '-v help' for level info" << endl <<
                     "--printexpire                  List of auto-expire programs" << endl <<
                     "--printsched                   Upcoming scheduled programs" << endl <<
+                    "--testsched                    Test run scheduler (ignore existing schedule)" << endl <<
                     "--resched                      Force the scheduler to update" << endl <<
+                    "--nosched                      Do not perform any scheduling" << endl <<
+                    "--nojobqueue                   Do not start the JobQueue" << endl <<
+                    "--nohousekeeper                Do not start the Housekeeper" << endl <<
+                    "--noautoexpire                 Do not start the AutoExpire thread" << endl <<
                     "--version                      Version information" << endl;
-            return -1;
+            return BACKEND_EXIT_INVALID_CMDLINE;
         }
     }
 
-    int logfd = -1;
-
-    if (logfile != "")
+    if (logfile != "" )
     {
-        logfd = open(logfile.ascii(), O_WRONLY|O_CREAT|O_APPEND|O_SYNC, 0664);
-         
-        if (logfd < 0)
-        {
-            perror(logfile.ascii());
-            cerr << "Error opening logfile\n";
-            return -1;
-        }
+        if (log_rotate(1) < 0)
+            cerr << "cannot open logfile; using stdout/stderr" << endl;
+        else
+            signal(SIGHUP, &log_rotate_handler);
     }
     
     ofstream pidfs;
@@ -397,7 +446,7 @@ int main(int argc, char **argv)
         {
             perror(pidfile.ascii());
             cerr << "Error opening pidfile";
-            return -1;
+            return BACKEND_EXIT_OPENING_PIDFILE_ERROR;
         }
     }
 
@@ -408,7 +457,7 @@ int main(int argc, char **argv)
         if (daemon(0, 1) < 0)
         {
             perror("daemon");
-            return -1;
+            return BACKEND_EXIT_DEAMONIZING_ERROR;
         }
 
 
@@ -418,84 +467,40 @@ int main(int argc, char **argv)
         pidfs.close();
     }
 
-    if (logfd != -1)
+    gContext = NULL;
+    gContext = new MythContext(MYTH_BINARY_VERSION);
+    if (!gContext->Init(false))
     {
-        // Send stdout and stderr to the logfile
-        dup2(logfd, 1);
-        dup2(logfd, 2);
-
-        // Close the unduplicated logfd
-        if (logfd != 1 && logfd != 2)
-            close(logfd);
+        VERBOSE(VB_IMPORTANT, "Failed to init MythContext, exiting.");
+        return BACKEND_EXIT_NO_MYTHCONTEXT;
     }
-
-    gContext = new MythContext(MYTH_BINARY_VERSION, false);
     gContext->SetBackend(true);
 
-    QSqlDatabase *db = QSqlDatabase::addDatabase("QMYSQL3");
-    if (!db)
-    {
-        cerr << "Couldn't connect to database\n";
-        return -1;
-    }
-
-    QSqlDatabase *subthread = QSqlDatabase::addDatabase("QMYSQL3", "SUBDB");
-    if (!subthread)
-    {
-        cerr << "Couldn't connect to database\n";
-        return -1;
-    }
-
-    QSqlDatabase *expthread = QSqlDatabase::addDatabase("QMYSQL3", "EXPDB");
-    if (!expthread)
-    {
-        cerr << "Couldn't connect to database\n";
-        return -1;
-    }
-
-    QSqlDatabase *hkthread = QSqlDatabase::addDatabase("QMYSQL3", "HKDB");
-    if (!hkthread)
-    {
-        cerr << "Couldn't connect to database\n";
-        return -1;
-    }
-
-    QSqlDatabase *jobthread = QSqlDatabase::addDatabase("QMYSQL3", "JOBDB");
-    if (!jobthread)
-    {
-        cerr << "Couldn't connect to database\n";
-        return -1;
-    }
-
-    QSqlDatabase *msdb = QSqlDatabase::addDatabase("QMYSQL3", "MSDB");
-    if (!msdb)
-    {
-        cerr << "Couldn't connect to database\n";
-        return -1;
-    }
-
-    if (!gContext->OpenDatabase(db, !daemonize && (logfile == "")) ||
-        !gContext->OpenDatabase(subthread, false) ||
-        !gContext->OpenDatabase(expthread, false) ||
-        !gContext->OpenDatabase(hkthread, false) ||
-        !gContext->OpenDatabase(jobthread, false) ||
-        !gContext->OpenDatabase(msdb, false))
-    {
-        cerr << "Couldn't open database\n";
-        return -1;
-    }
-
     close(0);
 
+    if (settingsOverride.size())
+    {
+        QMap<QString, QString>::iterator it;
+        for (it = settingsOverride.begin(); it != settingsOverride.end(); ++it)
+        {
+            VERBOSE(VB_IMPORTANT, QString("Setting '%1' being forced to '%2'")
+                                          .arg(it.key()).arg(it.data()));
+            gContext->OverrideSettingForSession(it.key(), it.data());
+        }
+    }
+
+    gContext->ActivateSettingsCache(false);
     if (!UpgradeTVDatabaseSchema())
     {
         VERBOSE(VB_IMPORTANT, "Couldn't upgrade database to new schema");
-        return -1;
+        return BACKEND_EXIT_DB_OUTOFDATE;
     }    
+    gContext->ActivateSettingsCache(true);
 
     if (printsched || testsched)
     {
-        sched = new Scheduler(false, &tvList, db);
+        gContext->SetBackend(false);
+        sched = new Scheduler(false, &tvList);
         if (!testsched && gContext->ConnectToMasterServer())
         {
             cout << "Retrieving Schedule from Master backend.\n";
@@ -512,32 +517,34 @@ int main(int argc, char **argv)
         print_verbose_messages |= VB_SCHEDULE;
         sched->PrintList(true);
         cleanup();
-        exit(0);
+        return BACKEND_EXIT_OK;
     }
 
     if (resched)
     {
         gContext->SetBackend(false);
 
+        bool ok = false;
         if (gContext->ConnectToMasterServer())
         {
             VERBOSE(VB_IMPORTANT, "Connected to master for reschedule");
             ScheduledRecording::signalChange(-1);
+            ok = true;
         }
         else
             VERBOSE(VB_IMPORTANT, "Cannot connect to master for reschedule");
 
         cleanup();
-        exit(0);
+        return (ok) ? BACKEND_EXIT_OK : BACKEND_EXIT_NO_CONNECT;
     }
 
     if (printexpire)
     {
-        expirer = new AutoExpire(false, false, db);
+        expirer = new AutoExpire(false, false);
         expirer->FillExpireList();
         expirer->PrintExpireList();
         cleanup();
-        exit(0);
+        return BACKEND_EXIT_OK;
     }
 
     int port = gContext->GetNumSetting("BackendServerPort", 6543);
@@ -553,7 +560,7 @@ int main(int argc, char **argv)
         cerr << "No setting found for this machine's BackendServerIP.\n"
              << "Please run setup on this machine and modify the first page\n"
              << "of the general settings.\n";
-        exit(3);
+        return BACKEND_EXIT_NO_IP_ADDRESS;
     }
 
     if (masterip == myip)
@@ -564,8 +571,8 @@ int main(int argc, char **argv)
         ismaster = true;
 
         if (nosched)
-            cerr << "******** The Scheduler has been DISABLED with "
-                    "the --nosched option ********\n";
+            cerr << "********** The Scheduler has been DISABLED with "
+                    "the --nosched option **********\n";
     }
     else
     {
@@ -573,28 +580,70 @@ int main(int argc, char **argv)
         gContext->LogEntry("mythbackend", LP_INFO,
                            "MythBackend started as a slave backend", "");
     }
- 
-    bool runsched = setupTVs(ismaster);
+
+    // Get any initial housekeeping done before we fire up anything else
+    if (nohousekeeper)
+        cerr << "****** The Housekeeper has been DISABLED with "
+                "the --nohousekeeper option ******\n";
+    else
+        housekeeping = new HouseKeeper(true, ismaster);
+
+    bool fatal_error = false;
+    bool runsched = setupTVs(ismaster, fatal_error);
+    if (fatal_error)
+        return BACKEND_EXIT_CAP_CARD_SETUP_ERROR;
 
     if (ismaster && runsched && !nosched)
     {
-        QSqlDatabase *scdb = QSqlDatabase::database("SUBDB");
-        sched = new Scheduler(true, &tvList, scdb);
+        sched = new Scheduler(true, &tvList);
     }
 
-    QSqlDatabase *expdb = QSqlDatabase::database("EXPDB");
-    expirer = new AutoExpire(true, ismaster, expdb);
+    if (noexpirer)
+        cerr << "********* Auto-Expire has been DISABLED with "
+                "the --noautoexpire option ********\n";
+    else
+        expirer = new AutoExpire(true, ismaster);
 
-    QSqlDatabase *hkdb = QSqlDatabase::database("HKDB");
-    housekeeping = new HouseKeeper(true, ismaster, hkdb);
+    if (nojobqueue)
+        cerr << "********* The JobQueue has been DISABLED with "
+                "the --nojobqueue option *********\n";
+    else
+        jobqueue = new JobQueue(ismaster);
 
-    QSqlDatabase *jobdb = QSqlDatabase::database("JOBDB");
-    jobqueue = new JobQueue(ismaster, jobdb);
+    // Initialize & Start the Mini HttpServer
+    VERBOSE(VB_IMPORTANT, "Main::Starting HttpServer");
 
-    VERBOSE(VB_ALL, QString("%1 version: %2 www.mythtv.org")
+    g_pHttpServer = new HttpServer(statusport);
+
+    if (!g_pHttpServer->ok())
+    { 
+        VERBOSE(VB_IMPORTANT, "Main::HttpServer Create Error");
+        // exit(BACKEND_BUGGY_EXIT_NO_BIND_STATUS);
+    }
+
+    VERBOSE(VB_IMPORTANT, "Main::Registering HttpStatus Extension");
+
+    g_pHttpServer->RegisterExtension(new HttpStatus(&tvList, sched, expirer, ismaster ));
+
+    // Start UPnP Services For Master Backends Only
+    if (ismaster)
+    {
+        g_pUPnp = new UPnp(ismaster, g_pHttpServer);
+
+        VERBOSE(VB_UPNP, "Main::Registering UPnpCDSTv Extension");
+
+        g_pUPnp->RegisterExtension(new UPnpCDSTv());
+
+        VERBOSE(VB_UPNP, "Main::Registering UPnpCDSMusic Extension");
+
+        g_pUPnp->RegisterExtension(new UPnpCDSMusic());
+    }
+    // End uPnP &  Mini HttpServer Initialization
+
+    VERBOSE(VB_IMPORTANT, QString("%1 version: %2 www.mythtv.org")
                             .arg(binname).arg(MYTH_BINARY_VERSION));
 
-    VERBOSE(VB_ALL, QString("Enabled verbose msgs :%1").arg(verboseString));
+    VERBOSE(VB_IMPORTANT, QString("Enabled verbose msgs: %1").arg(verboseString));
 
     lockfile_location = gContext->GetSetting("RecordFilePrefix") + "/nfslockfile.lock";
 
@@ -613,11 +662,11 @@ int main(int argc, char **argv)
                  << "Be sure that \'" << gContext->GetSetting("RecordFilePrefix")
                  << "\' exists and that both \nthe directory and that "
                  << "file are writeable by this user.\n";
-            return -1;
+            return BACKEND_EXIT_OPENING_VLOCKFILE_ERROR;
         }
     }
 
-    new MainServer(ismaster, port, statusport, &tvList, msdb, sched);
+    new MainServer(ismaster, port, statusport, &tvList, sched, expirer);
 
     if (ismaster)
     {
@@ -625,7 +674,7 @@ int main(int argc, char **argv)
             = gContext->GetSetting("WOLslaveBackendsCommand","");
         if (!WOLslaveBackends.isEmpty())
         {
-            VERBOSE(VB_ALL, "Waking slave Backends now.");
+            VERBOSE(VB_IMPORTANT, "Waking slave Backends now.");
             system(WOLslaveBackends.ascii());
         }
     }
@@ -635,5 +684,7 @@ int main(int argc, char **argv)
     gContext->LogEntry("mythbackend", LP_INFO, "MythBackend exiting", "");
     cleanup();
 
-    return 0;
+    return BACKEND_EXIT_OK;
 }
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
